@@ -44,7 +44,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_OTHER
-static const char rcsid[] = "$Id: auth_other.c,v 1.14 2001/10/18 10:11:29 chris Exp $";
+static const char rcsid[] = "$Id: auth_other.c,v 1.23 2002/03/18 23:47:33 chris Exp $";
 
 #include <sys/types.h>
 
@@ -82,8 +82,7 @@ struct timeval authchild_timeout;
 volatile int authchild_wr = -1, authchild_rd = -1;
 
 /* dump:
- * Debugging method.
- */
+ * Debugging method. */
 void dump(unsigned char *b, size_t len) {
     unsigned char *p;
     char *q;
@@ -95,14 +94,17 @@ void dump(unsigned char *b, size_t len) {
             q += 4;
         }
     *q = 0;
-    print_log(LOG_INFO, "dump %s", str);
+    log_print(LOG_INFO, "dump %s", str);
     xfree(str);
 }
 
+static void tvsub(struct timeval *t1, const struct timeval *t2);
+static void tvadd(struct timeval *t1, const struct timeval *t2);
+static int tvcmp(const struct timeval *t1, const struct timeval *t2);
+
 /* tvadd:
- * t1 += t2 on timevals.
- */
-void tvadd(struct timeval *t1, const struct timeval *t2) {
+ * t1 += t2 on timevals. */
+static void tvadd(struct timeval *t1, const struct timeval *t2) {
     t1->tv_sec  += t2->tv_sec;
     t1->tv_usec += t2->tv_usec;
     while (t1->tv_usec > 1000000) {
@@ -112,9 +114,8 @@ void tvadd(struct timeval *t1, const struct timeval *t2) {
 }
 
 /* tvsub:
- * t1 -= t2 on timevals.
- */
-void tvsub(struct timeval *t1, const struct timeval *t2) {
+ * t1 -= t2 on timevals. */
+static void tvsub(struct timeval *t1, const struct timeval *t2) {
     t1->tv_sec  -= t2->tv_sec;
     t1->tv_usec -= t2->tv_usec;
     while (t1->tv_usec < 0) {
@@ -124,9 +125,8 @@ void tvsub(struct timeval *t1, const struct timeval *t2) {
 }
 
 /* tvcmp:
- * Is t1 before (-1), after (1), or the same as (0) t2?
- */
-int tvcmp(const struct timeval *t1, const struct timeval *t2) {
+ * Is t1 before (-1), after (1), or the same as (0) t2? */
+static int tvcmp(const struct timeval *t1, const struct timeval *t2) {
     if (t1->tv_sec < t2->tv_sec) return -1;
     else if (t1->tv_sec > t2->tv_sec) return 1;
     else {
@@ -138,26 +138,26 @@ int tvcmp(const struct timeval *t1, const struct timeval *t2) {
 
 /* auth_other_start_child:
  * Start the authentication program, setting up pipes on which to talk to it.
- * Returns 1 on success or 0 on failure.
- */
+ * Returns 1 on success or 0 on failure. */
 int auth_other_start_child() {
     int p1[2], p2[2];
-    char *argv[2] = {auth_program, NULL};
+    char *argv[2] = {0};
     char *envp[3] = {"PATH=/bin",  /* XXX path? */
                      "TPOP3D_CONTEXT=auth_other", NULL};
 
+    argv[0] = auth_program;
+
     /* Generate pipes to talk to the child. */
     if (pipe(p1) == -1 || pipe(p2) == -1) {
-        print_log(LOG_ERR, "auth_other_start_child: pipe: %m");
+        log_print(LOG_ERR, "auth_other_start_child: pipe: %m");
         return 0;
     }
 
     /* p1[0] becomes the child's standard input.
      * p2[1] becomes the child's standard output.
-     * We want p1[1] to be non-blocking.
-     */
+     * We want p1[1] to be non-blocking. */
     if (fcntl(p1[1], F_SETFL, O_NONBLOCK) == -1) {
-        print_log(LOG_ERR, "auth_other_start_child: fcntl: %m");
+        log_print(LOG_ERR, "auth_other_start_child: fcntl: %m");
         close(p1[0]); close(p1[1]); close(p2[0]); close(p2[1]);
         return 0;
     }
@@ -165,15 +165,15 @@ int auth_other_start_child() {
     switch (authchild_pid = fork()) {
         case 0:
             if (setgid(authchild_gid) == -1) {
-                print_log(LOG_ERR, "auth_other_start_child: setgid(%d): %m", (int)authchild_gid);
+                log_print(LOG_ERR, "auth_other_start_child: setgid(%d): %m", (int)authchild_gid);
                 _exit(1);
             } else if (setuid(authchild_uid) == -1) {
-                print_log(LOG_ERR, "auth_other_start_child: setuid(%d): %m", (int)authchild_uid);
+                log_print(LOG_ERR, "auth_other_start_child: setuid(%d): %m", (int)authchild_uid);
                 _exit(1);
             }
             
             /* Child. */
-            close(0); close(1); close(2); 
+            close(0); close(1); close(2);
 
             /* Set up standard input and output. */
             dup2(p1[0], 0); dup2(p2[1], 1); 
@@ -188,7 +188,7 @@ int auth_other_start_child() {
             
         case -1:
             /* Error. */
-            print_log(LOG_ERR, "auth_other_start_child: fork: %m");
+            log_print(LOG_ERR, "auth_other_start_child: fork: %m");
 
             close(p1[0]); close(p1[1]); close(p2[0]); close(p2[1]);
             return 0;
@@ -201,16 +201,17 @@ int auth_other_start_child() {
             close(p1[0]);
             close(p2[1]);
 
-            print_log(LOG_INFO, "auth_other_start_child: started authentication child `%s'", auth_program);
+            log_print(LOG_INFO, "auth_other_start_child: started authentication child `%s'", auth_program);
 
             return 1;
     }
+
+    return 0; /* NOTREACHED */
 }
 
 /* auth_other_kill_child:
  * Kill the authentication child, with SIGTERM at first and then with added
- * SIGKILL-shaped violence if that fails.
- */
+ * SIGKILL-shaped violence if that fails. */
 void auth_other_kill_child() {
     struct timeval deadline, now;
     if (authchild_pid == 0) return; /* Already dead. */
@@ -228,7 +229,7 @@ void auth_other_kill_child() {
     }
 
     if (authchild_pid) {
-        print_log(LOG_WARNING, _("auth_other_kill_child: child failed to die; killing with SIGKILL"));
+        log_print(LOG_WARNING, _("auth_other_kill_child: child failed to die; killing with SIGKILL"));
         kill(authchild_pid, SIGKILL);
         /* Assume this works; it ought to! */
     }
@@ -237,40 +238,38 @@ void auth_other_kill_child() {
 /* auth_other_init:
  * Initialise the authentication driver for external programs. This starts the
  * program specified by the auth-other-program config directive, running under
- * the user and group ids in auth-other-user and auth-other-group.
- */
+ * the user and group ids in auth-other-user and auth-other-group. */
 extern stringmap config;    /* in main.c */
 int auth_other_init() {
-    item *I;
+    char *s;
     float f;
 
-    if (!(I = stringmap_find(config, "auth-other-program"))) {
-        print_log(LOG_ERR, _("auth_other_init: no program specified"));
+    if (!(s = config_get_string("auth-other-program"))) {
+        log_print(LOG_ERR, _("auth_other_init: no program specified"));
         return 0;
     } else {
         struct stat st;
-        auth_program = (char*)I->v;
+        auth_program = s;
         if (*auth_program != '/') {
-            print_log(LOG_ERR, _("auth_other_init: auth-program %s should be an absolute path"), auth_program);
+            log_print(LOG_ERR, _("auth_other_init: auth-program %s must be an absolute path"), auth_program);
             return 0;
         } else if (stat(auth_program, &st) == -1) {
-            print_log(LOG_ERR, _("auth_other_init: auth-program %s: %m"), auth_program);
+            log_print(LOG_ERR, _("auth_other_init: auth-program %s: %m"), auth_program);
             return 0;
         }
         /* XXX should fail if it turns out that the program is not executable
-         * by the given group and user; but this is a pain to work out.
-         */
+         * by the given group and user; but this is a pain to work out. */
     }
 
     /* Find out the timeout for talking to the program. */
     switch (config_get_float("auth-other-timeout", &f)) {
         case -1:
-            print_log(LOG_ERR, _("auth_other_init: value given for auth-other-timeout does not make sense"));
+            log_print(LOG_ERR, _("auth_other_init: value given for auth-other-timeout does not make sense"));
             return 0;
 
         case 1:
             if (f < 0.0 || f > 10.0) {
-                print_log(LOG_ERR, _("auth_other_init: value %f for auth-other-timeout is out of range"), f);
+                log_print(LOG_ERR, _("auth_other_init: value %f for auth-other-timeout is out of range"), f);
                 return 0;
             }
             break;
@@ -283,43 +282,48 @@ int auth_other_init() {
     authchild_timeout.tv_usec = (long)((f - floor(f)) * 1e6);
 
     /* Find out user and group under which program will run. */
-    if (!(I = stringmap_find(config, "auth-other-user"))) {
-        print_log(LOG_ERR, _("auth_other_init: no user specified"));
+    if (!(s = config_get_string("auth-other-user"))) {
+        log_print(LOG_ERR, _("auth_other_init: no user specified"));
         return 0;
-    } else if (!parse_uid(I->v, &authchild_uid)) {
-        print_log(LOG_ERR, _("auth_other_init: auth-other-user directive `%s' does not make sense"), I->v);
+    } else if (!parse_uid(s, &authchild_uid)) {
+        log_print(LOG_ERR, _("auth_other_init: auth-other-user directive `%s' does not make sense"), s);
         return 0;
     }
 
-    if (!(I = stringmap_find(config, "auth-other-group"))) {
-        print_log(LOG_ERR, _("auth_other_init: no group specified"));
+    if (!(s = config_get_string("auth-other-group"))) {
+        log_print(LOG_ERR, _("auth_other_init: no group specified"));
         return 0;
-    } else if (!parse_gid(I->v, &authchild_gid)) {
-        print_log(LOG_ERR, _("auth_other_init: auth-other-group directive `%s' does not make sense"), I->v);
+    } else if (!parse_gid(s, &authchild_gid)) {
+        log_print(LOG_ERR, _("auth_other_init: auth-other-group directive `%s' does not make sense"), s);
         return 0;
     }
 
-    print_log(LOG_INFO, "auth_other_init: %s: will run as uid %d, gid %d", auth_program, (int)authchild_uid, (int)authchild_gid);
+    log_print(LOG_INFO, "auth_other_init: %s: will run as uid %d, gid %d", auth_program, (int)authchild_uid, (int)authchild_gid);
 
     if (!auth_other_start_child()) {
-        print_log(LOG_ERR, _("auth_other_init: failed to start authentication child for first time"));
+        log_print(LOG_ERR, _("auth_other_init: failed to start authentication child for first time"));
         return 0;
     }
 
     return 1;
 }
 
+/* auth_other_postfork:
+ * Post-fork cleanup: close our copies of the file descriptors. */
+void auth_other_postfork() {
+    close(authchild_wr);
+    close(authchild_rd);
+}
+
 /* auth_other_close:
- * Shut down the authentication driver, killing the external program.
- */
+ * Shut down the authentication driver, killing the external program. */
 void auth_other_close() {
     auth_other_kill_child();
 }
 
 /* auth_other_send_request:
  * Send the auth child a request consisting of several key/value pairs, as
- * above. Returns 1 on success or 0 on failure.
- */
+ * above. Returns 1 on success or 0 on failure. */
 int auth_other_send_request(const int nvars, ...) {
     va_list ap;
     int i, ret = 0;
@@ -348,20 +352,19 @@ int auth_other_send_request(const int nvars, ...) {
 
     ++nn; /* Terminating \0 */
     if (nn > sizeof(buffer)) {
-        print_log(LOG_ERR, _("auth_other_send_request: total size of request would exceed %d bytes"), sizeof(buffer));
+        log_print(LOG_ERR, _("auth_other_send_request: total size of request would exceed %d bytes"), sizeof(buffer));
         goto fail;
     }
 
     /* Since write operations are atomic, this will either succeed entirely or
      * fail. In the latter case, it may be with EAGAIN because the child
-     * process is blocking; we interpret this as a protocol error.
-     */
+     * process is blocking; we interpret this as a protocol error. */
     if (try_write(authchild_wr, buffer, nn)) ret = 1;
     else {
         if (errno == EAGAIN)
-            print_log(LOG_ERR, _("auth_other_send_request: write: write on pipe blocked; killing child"));
+            log_print(LOG_ERR, _("auth_other_send_request: write: write on pipe blocked; killing child"));
         else
-            print_log(LOG_ERR, _("auth_other_send_request: write: %m; killing child"));
+            log_print(LOG_ERR, _("auth_other_send_request: write: %m; killing child"));
         auth_other_kill_child();
     }
     
@@ -373,8 +376,7 @@ fail:
 
 /* auth_other_recv_response:
  * Receive a response from the auth child, as above. Returns a stringmap of
- * responses on success, or NULL on failure.
- */
+ * responses on success, or NULL on failure. */
 stringmap auth_other_recv_response() {
     stringmap S = NULL;
     char buffer[MAX_DATA_SIZE], ends[2] = {0, 0};
@@ -396,7 +398,7 @@ stringmap auth_other_recv_response() {
         FD_SET(authchild_rd, &readfds);
         gettimeofday(&tt, NULL);
         if (tvcmp(&deadline, &tt) == -1) {
-            print_log(LOG_ERR, _("auth_other_recv_response: timed out waiting for a response; killing child"));
+            log_print(LOG_ERR, _("auth_other_recv_response: timed out waiting for a response; killing child"));
             goto fail;
         }
         tvsub(&timeout, &tt);
@@ -407,19 +409,19 @@ stringmap auth_other_recv_response() {
 
                 switch (rr) {
                     case 0:
-                        print_log(LOG_ERR, _("auth_other_recv_response: read: child closed pipe; killing child"));
+                        log_print(LOG_ERR, _("auth_other_recv_response: read: child closed pipe; killing child"));
                         goto fail;
 
                     case -1:
                         if (errno != EINTR) {
-                            print_log(LOG_ERR, _("auth_other_recv_response: read: %m; killing child"));
+                            log_print(LOG_ERR, _("auth_other_recv_response: read: %m; killing child"));
                             goto fail;
                         } else break;
 
                     default:
                         p += rr;
                         if (p == buffer + sizeof(buffer)) {
-                            print_log(LOG_ERR, _("auth_other_recv_response: total size of response exceeds %d bytes; killing child"), sizeof(buffer));
+                            log_print(LOG_ERR, _("auth_other_recv_response: total size of response exceeds %d bytes; killing child"), sizeof(buffer));
                             goto fail;
                         }
                 }
@@ -427,7 +429,7 @@ stringmap auth_other_recv_response() {
 
             case -1:
                 if (errno != EINTR) {
-                    print_log(LOG_ERR, _("auth_other_recv_repsonse: select: %m; killing child"));
+                    log_print(LOG_ERR, _("auth_other_recv_repsonse: select: %m; killing child"));
                     goto fail;
                 }
 
@@ -437,14 +439,12 @@ stringmap auth_other_recv_response() {
     } while (p < buffer + 2 || memcmp(p - 2, ends, 2) != 0); /* Now see whether we have some valid data. */
 
     /* Try to interpret the passed data. We want to find pairs of
-     * \0-terminated strings and put them into the stringmap.
-     */
+     * \0-terminated strings and put them into the stringmap. */
     S = stringmap_new();
     
     /* q points to the beginning of a key, r to the end of the key, and s to
      * the end of the value, so that [q...r] is the key and [r + 1...s] is the
-     * value.
-     */
+     * value. */
     q = buffer;
     while (*q && S) {
         r = memchr(q, 0, p - q);
@@ -453,13 +453,13 @@ stringmap auth_other_recv_response() {
         s = memchr(r + 1, 0, p - (r + 1));
         if (!s) goto formaterror;
 
-        stringmap_insert(S, q, item_ptr(strdup(r + 1)));
+        stringmap_insert(S, q, item_ptr(xstrdup(r + 1)));
 
         q = s + 1;
         continue;
 
 formaterror:
-        print_log(LOG_ERR, _("auth_other_recv_response: response data not correctly formatted; killing child"));
+        log_print(LOG_ERR, _("auth_other_recv_response: response data not correctly formatted; killing child"));
         stringmap_delete_free(S);
         S = NULL;
     }
@@ -470,11 +470,10 @@ fail:
 }
 
 /* auth_other_new_apop:
- * Attempt to authenticate a user using APOP, via the child program.
- */
-authcontext auth_other_new_apop(const char *name, const char *timestamp, const unsigned char *digest, const char *host) {
-#define MISSING(k)     do { print_log(LOG_ERR, _("auth_other_new_apop: missing key `%s' in response"), (k)); goto fail; } while(0)
-#define INVALID(k, v)  do { print_log(LOG_ERR, _("auth_other_new_apop: invalid value `%s' for key `%s' in response"), (v), (k)); goto fail; } while(0)
+ * Attempt to authenticate a user using APOP, via the child program. */
+authcontext auth_other_new_apop(const char *name, const char *local_part, const char *domain, const char *timestamp, const unsigned char *digest, const char *host) {
+#define MISSING(k)     do { log_print(LOG_ERR, _("auth_other_new_apop: missing key `%s' in response"), (k)); goto fail; } while(0)
+#define INVALID(k, v)  do { log_print(LOG_ERR, _("auth_other_new_apop: invalid value `%s' for key `%s' in response"), (v), (k)); goto fail; } while(0)
     char digeststr[33];
     char *p;
     const unsigned char *q;
@@ -486,12 +485,17 @@ authcontext auth_other_new_apop(const char *name, const char *timestamp, const u
     
     for (p = digeststr, q = digest; q < digest + 16; p += 2, ++q)
         sprintf(p, "%02x", (unsigned int)*q);
-    if (!auth_other_send_request(5, "method", "APOP", "user", name, "timestamp", timestamp, "digest", digeststr, "clienthost", host)
-        || !(S = auth_other_recv_response()))
+    if (local_part && domain) {
+        if (!auth_other_send_request(7, "method", "APOP", "user", name, "local_part", local_part, "domain", domain, "timestamp", timestamp, "digest", digeststr, "clienthost", host))
+            return NULL;
+    } else if (!auth_other_send_request(5, "method", "APOP", "user", name, "timestamp", timestamp, "digest", digeststr, "clienthost", host))
+        return NULL;
+
+    if (!(S = auth_other_recv_response()))
         return NULL;
 
     I = stringmap_find(S, "logmsg");
-    if (I) print_log(LOG_INFO, "auth_other_new_apop: child: %s", (char*)I->v);
+    if (I) log_print(LOG_INFO, "auth_other_new_apop: child: %s", (char*)I->v);
 
     I = stringmap_find(S, "result");
     if (!I) MISSING("result");
@@ -522,7 +526,7 @@ authcontext auth_other_new_apop(const char *name, const char *timestamp, const u
         I = stringmap_find(S, "domain");
         if (I) domain = (char*)I->v;
 
-        a = authcontext_new(uid, gid, mboxdrv, mailbox, pw->pw_dir, domain);
+        a = authcontext_new(uid, gid, mboxdrv, mailbox, pw->pw_dir);
     } else if (strcmp((char*)I->v, "NO") != 0) INVALID("result", (char*)I->v);
         
 fail:
@@ -533,23 +537,27 @@ fail:
 }
 
 /* auth_other_new_user_pass:
- * Attempt to authenticate a user using USER/PASS, via the child program.
- */
-authcontext auth_other_new_user_pass(const char *user, const char *pass, const char *host) {
-#define MISSING(k)     do { print_log(LOG_ERR, _("auth_other_new_user_pass: missing key `%s' in response"), (k)); goto fail; } while(0)
-#define INVALID(k, v)  do { print_log(LOG_ERR, _("auth_other_new_user_pass: invalid value `%s' for key `%s' in response"), (v), (k)); goto fail; } while(0)
+ * Attempt to authenticate a user using USER/PASS, via the child program. */
+authcontext auth_other_new_user_pass(const char *user, const char *local_part, const char *domain, const char *pass, const char *host) {
+#define MISSING(k)     do { log_print(LOG_ERR, _("auth_other_new_user_pass: missing key `%s' in response"), (k)); goto fail; } while(0)
+#define INVALID(k, v)  do { log_print(LOG_ERR, _("auth_other_new_user_pass: invalid value `%s' for key `%s' in response"), (v), (k)); goto fail; } while(0)
     stringmap S;
     item *I;
     authcontext a = NULL;
 
     if (!authchild_pid) auth_other_start_child();
 
-    if (!auth_other_send_request(4, "method", "PASS", "user", user, "pass", pass, "clienthost", host)
-        || !(S = auth_other_recv_response()))
+    if (local_part && domain) {
+        if (!auth_other_send_request(6, "method", "PASS", "user", user, "local_part", local_part, "domain", domain, "pass", pass, "clienthost", host))
+            return NULL;
+    } else if (!auth_other_send_request(4, "method", "PASS", "user", user, "pass", pass, "clienthost", host))
+        return NULL;
+
+    if (!(S = auth_other_recv_response()))
         return NULL;
     
     I = stringmap_find(S, "logmsg");
-    if (I) print_log(LOG_INFO, "auth_other_new_user_pass: child: %s", (char*)I->v);
+    if (I) log_print(LOG_INFO, "auth_other_new_user_pass: child: %s", (char*)I->v);
 
     I = stringmap_find(S, "result");
     if (!I) MISSING("result");
@@ -580,7 +588,7 @@ authcontext auth_other_new_user_pass(const char *user, const char *pass, const c
         I = stringmap_find(S, "domain");
         if (I) domain = (char*)I->v;
 
-        a = authcontext_new(uid, gid, mboxdrv, mailbox, pw->pw_dir, domain);
+        a = authcontext_new(uid, gid, mboxdrv, mailbox, pw->pw_dir);
     } else if (strcmp((char*)I->v, "NO") != 0) INVALID("result", (char*)I->v);
         
 fail:
@@ -590,4 +598,23 @@ fail:
 #undef INVALID
 }
 
+/* auth_other_onlogin:
+ * Pass details of a successful login to the authentication program. */
+void auth_other_onlogin(const authcontext A, const char *host) {
+    stringmap S;
+    item *I;
+
+    if (!authchild_pid) auth_other_start_child();
+
+    if (!auth_other_send_request(5, "method", "ONLOGIN", "user", A->user, "local_part", A->local_part, "domain", A->domain, "clienthost", host)
+        || !(S = auth_other_recv_response()))
+        return;
+    
+    I = stringmap_find(S, "logmsg");
+    if (I) log_print(LOG_INFO, "auth_other_new_user_pass: child: %s", (char*)I->v);
+        
+    stringmap_delete_free(S);
+}
+
+
 #endif /* AUTH_OTHER */
