@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
  *
- * $Id: auth_other.h,v 1.2 2001/04/26 23:30:03 chris Exp $
+ * $Id: auth_other.h,v 1.5 2002/02/24 18:07:52 chris Exp $
  *
  */
 
@@ -22,17 +22,16 @@
 #include "stringmap.h"
 
 /* auth_other.c */
-void tvsub(struct timeval *t1, const struct timeval *t2);
-void tvsub(struct timeval *t1, const struct timeval *t2);
-int tvcmp(const struct timeval *t1, const struct timeval *t2);
 int auth_other_start_child(void);
 void auth_other_kill_child(void);
 int auth_other_init(void);
+void auth_other_postfork(void);
 void auth_other_close(void);
 int auth_other_send_request(const int nvars, ...);
 stringmap auth_other_recv_response(void);
-authcontext auth_other_new_apop(const char *name, const char *timestamp, const unsigned char *digest);
-authcontext auth_other_new_user_pass(const char *user, const char *pass);
+authcontext auth_other_new_apop(const char *name, const char *local_part, const char *domain, const char *timestamp, const unsigned char *digest, const char *host);
+authcontext auth_other_new_user_pass(const char *user, const char *local_part, const char *domain, const char *pass, const char *host);
+void auth_other_onlogin(const authcontext A, const char *host);
 
 #endif /* AUTH_OTHER */
 
