@@ -4,7 +4,7 @@
  *
  * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
  *
- * $Id: auth_pam.h,v 1.6 2001/04/23 23:45:40 chris Exp $
+ * $Id: auth_pam.h,v 1.8 2002/02/24 18:07:52 chris Exp $
  *
  */
 
@@ -29,7 +29,7 @@
 #define AUTH_PAM_FACILITY       "tpop3d"
 
 int         auth_pam_init(void);
-authcontext auth_pam_new_user_pass(const char *user, const char *pass);
+authcontext auth_pam_new_user_pass(const char *user, const char *local_part, const char *domain, const char *pass, const char *host);
 void        auth_pam_close(void);
 
 #endif /* AUTH_PAM */
