@@ -11,7 +11,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_PASSWD
-static const char rcsid[] = "$Id: auth_passwd.c,v 1.13 2001/10/14 02:09:55 chris Exp $";
+static const char rcsid[] = "$Id: auth_passwd.c,v 1.16 2002/02/24 18:07:52 chris Exp $";
 
 #include <sys/types.h>
 
@@ -35,31 +35,27 @@ static const char rcsid[] = "$Id: auth_passwd.c,v 1.13 2001/10/14 02:09:55 chris
      
 #include "auth_passwd.h"
 #include "authswitch.h"
-#include "stringmap.h"
+#include "config.h"
 #include "util.h"
 #include "mailbox.h"
 
 /* auth_passwd_new_user_pass:
  * Attempt to authenticate user and pass using /etc/passwd or /etc/shadow,
- * as configured at compile-time.
- */
-extern stringmap config;
-
-authcontext auth_passwd_new_user_pass(const char *user, const char *pass, const char *host /* unused */) {
+ * as configured at compile-time. This is not a virtual-domains authenticator,
+ * so it only uses user. */
+authcontext auth_passwd_new_user_pass(const char *user, const char *local_part, const char *domain, const char *pass, const char *host /* unused */) {
     struct passwd *pw;
 #ifdef AUTH_PASSWD_SHADOW
     struct spwd *spw;
 #endif /* AUTH_PASSWD_SHADOW */
     char *user_passwd;
-    item *I;
+    char *s;
     int use_gid = 0;
     gid_t gid = 99;
     authcontext a = NULL;
-    const char *x;
 
     /* Check the this isn't a virtual-domain user. */
-    x = user + strcspn(user, "@%!");
-    if (*x) return NULL;
+    if (local_part) return NULL;
 
     pw = getpwnam(user);
     if (!pw) return NULL;
@@ -72,19 +68,18 @@ authcontext auth_passwd_new_user_pass(const char *user, const char *pass, const
 #endif /* AUTH_PASSWD_SHADOW */
 
     /* Obtain gid to use */
-    if ((I = stringmap_find(config, "auth-passwd-mail-group"))) {
-        if (!parse_gid((char*)I->v, &gid)) {
-            print_log(LOG_ERR, _("auth_passwd_new_user_pass: auth-passwd-mail-group directive `%s' does not make sense"), (char*)I->v);
+    if ((s = config_get_string("auth-passwd-mail-group"))) {
+        if (!parse_gid(s, &gid)) {
+            log_print(LOG_ERR, _("auth_passwd_new_user_pass: auth-passwd-mail-group directive `%s' does not make sense"), s);
             return NULL;
         }
         use_gid = 1;
     }
 
     /* Now we need to authenticate the user; we will leave finding the
-     * mailspool for later.
-     */
+     * mailspool for later. */
     if (!strcmp(crypt(pass, user_passwd), user_passwd)) {
-        a = authcontext_new(pw->pw_uid, use_gid ? gid : pw->pw_gid, NULL, NULL, pw->pw_dir, NULL);
+        a = authcontext_new(pw->pw_uid, use_gid ? gid : pw->pw_gid, NULL, NULL, pw->pw_dir);
     }
     
     return a;
