@@ -14,7 +14,7 @@
 
 #ifdef AUTH_PERL
 
-static const char rcsid[] = "$Id: auth_perl.c,v 1.12 2001/10/14 02:09:55 chris Exp $";
+static const char rcsid[] = "$Id: auth_perl.c,v 1.22 2002/03/22 19:37:56 chris Exp $";
 
 #include <sys/types.h>
 
@@ -23,6 +23,7 @@ static const char rcsid[] = "$Id: auth_perl.c,v 1.12 2001/10/14 02:09:55 chris E
 #include <syslog.h>
 #include <unistd.h>
 
+#include "config.h"
 #include "auth_perl.h"
 #include "stringmap.h"
 #include "util.h"
@@ -33,77 +34,76 @@ static const char rcsid[] = "$Id: auth_perl.c,v 1.12 2001/10/14 02:09:55 chris E
 #include <perl.h>
 #include <XSUB.h>
 
-static PerlInterpreter *auth_perl_interp;
-char *auth_perl_apop, *auth_perl_pass;      /* Names of functions we call. */
+static PerlInterpreter *perl_interp;
+char *apop_sub, *pass_sub, *onlogin_sub;    /* Names of functions we call. */
 
 #ifndef ERRSV
 #define ERRSV (GvSV(errgv))
 #endif
 
-/* xs_print_log:
- * Perl interface to tpop3d's logging.
- */
-XS(xs_print_log)
+/* xs_log_print:
+ * Perl interface to tpop3d's logging. */
+XS(xs_log_print)
 {
     dXSARGS;
     char  *str;
     STRLEN len;
     
-    if (items != 1) croak("Usage: TPOP3D::print_log(string)");
+    if (items != 1) croak("Usage: TPOP3D::log_print(string)");
 
     str = SvPV(ST(0), len);
-    print_log(LOG_INFO, "auth_perl: (perl code): %s", str);
+    log_print(LOG_INFO, "auth_perl: (perl code): %s", str);
 }
 
 /* xs_init:
- * Start up XS code in perl.
- */
+ * Start up XS code in perl. */
 extern void boot_DynaLoader(CV *cv);
 
 void xs_init(void) {
     char *file = __FILE__;
     newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
-    newXS("TPOP3D::print_log", xs_print_log, file);
+    newXS("TPOP3D::log_print", xs_log_print, file);
 }
 
 /* auth_perl_init:
- * Initialise the perl interpreter and run its startup code.
- */
+ * Initialise the perl interpreter and run its startup code. */
 extern stringmap config;    /* in main.c */
 
-int auth_perl_init() {
+int auth_perl_init(void) {
     dSP;
     int argc = 2;
     char *argv[3] = {"auth_perl", "/dev/null", NULL};
 /*   char *argv[4] = {"auth_perl", "-e", "$ENV{TPOP3D_CONTEXT} = 'auth_perl';", NULL}; */
-    item *I;
-    char *startupcode;
+    char *startupcode, *s;
     SV *sv;
     STRLEN len;
 
     /* Obtain perl startup code; this should probably be something like
-     * "do '/etc/tpop3d.pl';"
-     */
-    if (!(I = stringmap_find(config, "auth-perl-start"))) {
-        print_log(LOG_ERR, _("auth_perl_init: auth_perl enabled, but no startup code specified"));
+     * "do '/etc/tpop3d.pl';" */
+    if (!(s = config_get_string("auth-perl-start"))) {
+        log_print(LOG_ERR, _("auth_perl_init: auth_perl enabled, but no startup code specified"));
         return 0;
-    } else startupcode = I->v;
+    } else startupcode = s;
 
-    if ((I = stringmap_find(config, "auth-perl-apop"))) auth_perl_apop = I->v;
-    if ((I = stringmap_find(config, "auth-perl-pass"))) auth_perl_pass = I->v;
-    if (!auth_perl_apop && !auth_perl_pass) {
-        print_log(LOG_ERR, _("auth_perl_init: auth_perl enabled but no authenticator subroutines supplied"));
+    if ((s = config_get_string("auth-perl-apop")))
+        apop_sub = s;
+    if ((s = config_get_string("auth-perl-pass")))
+        pass_sub = s;
+    if ((s = config_get_string("auth-perl-onlogin")))
+        onlogin_sub = s;
+    if (!apop_sub && !pass_sub && !onlogin_sub) {
+        log_print(LOG_ERR, _("auth_perl_init: auth_perl enabled but no subroutines supplied"));
         return 0;
     }
 
     /* Put a useful string into the environment. */
-    putenv(strdup("TPOP3D_CONTEXT=auth_perl"));
+    putenv(xstrdup("TPOP3D_CONTEXT=auth_perl"));
 
     /* Create and start up perl interpreter. */
-    auth_perl_interp = perl_alloc();
-    perl_construct(auth_perl_interp);
-    perl_parse(auth_perl_interp, xs_init, argc, argv, 0);
-    perl_run(auth_perl_interp);
+    perl_interp = perl_alloc();
+    perl_construct(perl_interp);
+    perl_parse(perl_interp, xs_init, argc, argv, 0);
+    perl_run(perl_interp);
 
     /* Try to execute the startup code. */
     sv = newSVpv(startupcode, 0);
@@ -111,45 +111,49 @@ int auth_perl_init() {
     perl_eval_sv(sv, G_SCALAR | G_DISCARD | G_KEEPERR); /* XXX what do the options actually mean? */
     SvREFCNT_dec(sv);
     if (SvTRUE(ERRSV)) {
-        print_log(LOG_ERR, _("auth_perl_init: error executing perl start code: %s"), SvPV(ERRSV, len));
-        perl_destruct(auth_perl_interp);
-        perl_free(auth_perl_interp);
-        auth_perl_interp = NULL;
+        log_print(LOG_ERR, _("auth_perl_init: error executing perl start code: %s"), SvPV(ERRSV, len));
+        perl_destruct(perl_interp);
+        perl_free(perl_interp);
+        perl_interp = NULL;
         return 0;
     }
 
     return 1;
 }
 
+/* auth_perl_postfork:
+ * Post-fork cleanup. */
+void auth_perl_postfork(void) {
+    perl_interp = NULL; /* XXX */
+}
+
 /* auth_perl_close:
- * Shut down the perl interpreter.
- */
-void auth_perl_close() {
-    if (auth_perl_interp) {
+ * Shut down the perl interpreter. */
+void auth_perl_close(void) {
+    if (perl_interp) {
         /* There may be code to execute on shutdown. */
-        item *I;
-        if ((I = stringmap_find(config, "auth-perl-finish"))) {
+        char *s;
+        if ((s = config_get_string("auth-perl-finish"))) {
             dSP;
             SV *sv;
             STRLEN len;
-            sv = newSVpv(I->v, 0);
+            sv = newSVpv(s, 0);
             PUSHMARK(SP);
             perl_eval_sv(sv, G_SCALAR | G_DISCARD | G_KEEPERR);
             SvREFCNT_dec(sv);
             if (SvTRUE(ERRSV))
-                print_log(LOG_ERR, _("auth_perl_close: error executing perl finish code: %s"), SvPV(ERRSV, len));
+                log_print(LOG_ERR, _("auth_perl_close: error executing perl finish code: %s"), SvPV(ERRSV, len));
         }
-        perl_destruct(auth_perl_interp);
-        perl_free(auth_perl_interp);
-        auth_perl_interp = NULL;
+        perl_destruct(perl_interp);
+        perl_free(perl_interp);
+        perl_interp = NULL;
     }
 }
 
 /* auth_perl_callfn:
  * Calls a perl function, passing the parameters in as a reference to a hash,
  * expecting a reference to a hash to be returned; it converts this into a
- * stringmap and returns it to the caller.
- */
+ * stringmap and returns it to the caller. */
 stringmap auth_perl_callfn(const char *perlfn, const int nvars, ...) {
     dSP;
     HV *hash_in, *hash_out;
@@ -158,7 +162,7 @@ stringmap auth_perl_callfn(const char *perlfn, const int nvars, ...) {
     int i, items;
     stringmap s = NULL;
 
-    if (!auth_perl_interp) return NULL;
+    if (!perl_interp) return NULL;
     
     hash_in = newHV();
 
@@ -192,15 +196,15 @@ stringmap auth_perl_callfn(const char *perlfn, const int nvars, ...) {
     if (SvTRUE(ERRSV)) {
         /* Error. */
         STRLEN len;
-        print_log(LOG_ERR, _("auth_perl_callfn: perl function %s: %s"), perlfn, SvPV(ERRSV, len));
+        log_print(LOG_ERR, _("auth_perl_callfn: perl function %s: %s"), perlfn, SvPV(ERRSV, len));
     } else if (!SvOK(hashref_out)) {
         /* Other sort of error. */
-        print_log(LOG_ERR, _("auth_perl_callfn: perl function %s: failure return"), perlfn);
+        log_print(LOG_ERR, _("auth_perl_callfn: perl function %s: failure return"), perlfn);
     } else if (SvTYPE(SvRV(hashref_out)) != SVt_PVHV) {
         /* Yet a third sort of error. */
-        print_log(LOG_ERR, _("auth_perl_callfn: perl function %s: returned value was not a reference to a hash"), perlfn);
+        log_print(LOG_ERR, _("auth_perl_callfn: perl function %s: returned value was not a reference to a hash"), perlfn);
     } else {
-        /* Damn and all, it worked! (Maybe) */
+        /* Damn and all, it worked! (Maybe.) */
         char *key;
         I32 len;
         SV *val;
@@ -210,13 +214,14 @@ stringmap auth_perl_callfn(const char *perlfn, const int nvars, ...) {
         /* Transfer contents of hash into s. */
         hv_iterinit(hash_out);
         while ((val = hv_iternextsv(hash_out, &key, &len))) {
-            char *k = xmalloc(len + 1);
             STRLEN len2;
-            strcpy(k, key);
-            stringmap_insert(s, k, item_ptr(strdup(SvPV(val, len2))));
+            stringmap_insert(s, key, item_ptr(xstrdup(SvPV(val, len2))));
         }
+        SvREFCNT_dec(hashref_out);
     }
 
+    SvREFCNT_dec(hashref_in);
+    
     FREETMPS;
     LEAVE;
 
@@ -225,11 +230,10 @@ stringmap auth_perl_callfn(const char *perlfn, const int nvars, ...) {
 
 /* auth_perl_new_apop:
  * Attempt to authenticate a user using APOP, via a perl subroutine. Much like
- * auth_other_new_apop.
- */
-authcontext auth_perl_new_apop(const char *name, const char *timestamp, const unsigned char *digest, const char *host) {
-#define MISSING(k)     do { print_log(LOG_ERR, _("auth_perl_new_apop: missing key `%s' in response"), (k)); goto fail; } while(0)
-#define INVALID(k, v)  do { print_log(LOG_ERR, _("auth_perl_new_apop: invalid value `%s' for key `%s' in response"), (v), (k)); goto fail; } while(0)
+ * auth_other_new_apop. */
+authcontext auth_perl_new_apop(const char *name, const char *local_part, const char *domain, const char *timestamp, const unsigned char *digest, const char *host) {
+#define MISSING(k)     do { log_print(LOG_ERR, _("auth_perl_new_apop: missing key `%s' in response"), (k)); goto fail; } while(0)
+#define INVALID(k, v)  do { log_print(LOG_ERR, _("auth_perl_new_apop: invalid value `%s' for key `%s' in response"), (v), (k)); goto fail; } while(0)
     char digeststr[33];
     char *p;
     const unsigned char *q;
@@ -237,14 +241,20 @@ authcontext auth_perl_new_apop(const char *name, const char *timestamp, const un
     item *I;
     authcontext a = NULL;
  
+    if (!apop_sub)
+        return NULL;
+    
     for (p = digeststr, q = digest; q < digest + 16; p += 2, ++q)
         sprintf(p, "%02x", (unsigned int)*q);
-    if (!auth_perl_apop ||
-        !(S = auth_perl_callfn(auth_perl_apop, 5, "method", "APOP", "user", name, "timestamp", timestamp, "digest", digeststr, "clienthost", host)))
+
+    if (local_part && domain) {
+        if (!(S = auth_perl_callfn(apop_sub, 7, "method", "APOP", "user", name, "local_part", local_part, "domain", domain, "timestamp", timestamp, "digest", digeststr, "clienthost", host)))
+            return NULL;
+    } else if (!(S = auth_perl_callfn(apop_sub, 5, "method", "APOP", "user", name, "timestamp", timestamp, "digest", digeststr, "clienthost", host)))
         return NULL;
 
     I = stringmap_find(S, "logmsg");
-    if (I) print_log(LOG_INFO, "auth_perl_new_apop: (perl code): %s", (char*)I->v);
+    if (I) log_print(LOG_INFO, "auth_perl_new_apop: (perl code): %s", (char*)I->v);
 
     I = stringmap_find(S, "result");
     if (!I) MISSING("result");
@@ -275,7 +285,7 @@ authcontext auth_perl_new_apop(const char *name, const char *timestamp, const un
         I = stringmap_find(S, "domain");
         if (I) domain = (char*)I->v;
 
-        a = authcontext_new(uid, gid, mboxdrv, mailbox, pw->pw_dir, domain);
+        a = authcontext_new(uid, gid, mboxdrv, mailbox, pw->pw_dir);
     } else if (strcmp((char*)I->v, "NO") != 0) INVALID("result", (char*)I->v);
         
 fail:
@@ -286,20 +296,25 @@ fail:
 }
 
 /* auth_perl_new_user_pass:
- * Attempt to authenticate a user using USER/PASS, via a perl subroutine.
- */
-authcontext auth_perl_new_user_pass(const char *user, const char *pass, const char *host) {
-#define MISSING(k)     do { print_log(LOG_ERR, _("auth_perl_new_user_pass: missing key `%s' in response"), (k)); goto fail; } while(0)
-#define INVALID(k, v)  do { print_log(LOG_ERR, _("auth_perl_new_user_pass: invalid value `%s' for key `%s' in response"), (v), (k)); goto fail; } while(0)
+ * Attempt to authenticate a user using USER/PASS, via a perl subroutine. */
+authcontext auth_perl_new_user_pass(const char *user, const char *local_part, const char *domain, const char *pass, const char *host) {
+#define MISSING(k)     do { log_print(LOG_ERR, _("auth_perl_new_user_pass: missing key `%s' in response"), (k)); goto fail; } while(0)
+#define INVALID(k, v)  do { log_print(LOG_ERR, _("auth_perl_new_user_pass: invalid value `%s' for key `%s' in response"), (v), (k)); goto fail; } while(0)
     stringmap S;
     item *I;
     authcontext a = NULL;
 
-    if (!auth_perl_pass || !(S = auth_perl_callfn(auth_perl_pass, 4, "method", "PASS", "user", user, "pass", pass, "clienthost", host)))
+    if (!pass_sub)
+        return NULL;
+
+    if (local_part && domain) {
+        if (!(S = auth_perl_callfn(pass_sub, 6, "method", "PASS", "user", user, "local_part", local_part, "domain", domain, "pass", pass, "clienthost", host)))
+            return NULL;
+    } else if (!(S = auth_perl_callfn(pass_sub, 4, "method", "PASS", "user", user, "pass", pass, "clienthost", host)))
         return NULL;
     
     if ((I = stringmap_find(S, "logmsg")))
-        print_log(LOG_INFO, "auth_perl_new_user_pass: (perl code): %s", (char*)I->v);
+        log_print(LOG_INFO, "auth_perl_new_user_pass: (perl code): %s", (char*)I->v);
 
     if (!(I = stringmap_find(S, "result"))) MISSING("result");
     
@@ -329,7 +344,7 @@ authcontext auth_perl_new_user_pass(const char *user, const char *pass, const ch
         I = stringmap_find(S, "domain");
         if (I) domain = (char*)I->v;
 
-        a = authcontext_new(uid, gid, mboxdrv, mailbox, pw->pw_dir, domain);
+        a = authcontext_new(uid, gid, mboxdrv, mailbox, pw->pw_dir);
     } else if (strcmp((char*)I->v, "NO") != 0) INVALID("result", (char*)I->v);
         
 fail:
@@ -339,4 +354,19 @@ fail:
 #undef INVALID
 }
 
-#endif
+/* auth_perl_onlogin:
+ * Pass details of a successful login to a perl subroutine. */
+void auth_perl_onlogin(const authcontext A, const char *host) {
+    stringmap S;
+    item *I;
+
+    if (!onlogin_sub || !(S = auth_perl_callfn(onlogin_sub, 5, "method", "ONLOGIN", "user", A->user, "local_part", A->local_part, "domain", A->domain, "clienthost", host)))
+        return;
+    
+    if ((I = stringmap_find(S, "logmsg")))
+        log_print(LOG_INFO, "auth_perl_onlogin: (perl code): %s", (char*)I->v);
+
+    stringmap_delete_free(S);
+}
+
+#endif /* AUTH_PERL */
