@@ -9,7 +9,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: cfgdirectives.c,v 1.8 2001/09/29 17:08:05 chris Exp $";
+static const char rcsid[] = "$Id: cfgdirectives.c,v 1.13 2002/03/18 23:47:33 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -23,7 +23,9 @@ char *cfgdirectives[] = {
     "listen-address",
     "max-children",
     "append-domain",
+    "strip-domain",
     "timeout-seconds",
+    "log-facility",
     "mailbox",
 
 #ifdef AUTH_PAM
@@ -51,8 +53,26 @@ char *cfgdirectives[] = {
     "auth-mysql-mail-group",
     "auth-mysql-pass-query",
     "auth-mysql-apop-query",
+    "auth-mysql-onlogin-query",
 #endif /* AUTH_MYSQL */
 
+#ifdef AUTH_LDAP
+    "auth-ldap-enable",
+    "auth-ldap-url",
+    "auth-ldap-searchdn",
+    "auth-ldap-password",
+    "auth-ldap-use-tls",
+    "auth-ldap-filter",
+    "auth-ldap-scope",
+    "auth-ldap-mailbox",
+    "auth-ldap-mailbox-attr",
+    "auth-ldap-mboxtype-attr",
+    "auth-ldap-mail-user",
+    "auth-ldap-mail-user-attr",
+    "auth-ldap-mail-group",
+    "auth-ldap-mail-group-attr",
+#endif /* AUTH_LDAP */
+    
 #ifdef AUTH_OTHER
     "auth-other-enable",
     "auth-other-program",
@@ -67,6 +87,7 @@ char *cfgdirectives[] = {
     "auth-perl-finish",
     "auth-perl-apop",
     "auth-perl-pass",
+    "auth-perl-onlogin",
 #endif /* AUTH_PERL */
  
 #if defined(MBOX_BSD) && defined(MBOX_BSD_SAVE_INDICES)
