@@ -6,7 +6,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: connection.c,v 1.28 2001/10/18 10:10:50 chris Exp $";
+static const char rcsid[] = "$Id: connection.c,v 1.31 2002/03/18 23:47:33 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -35,8 +35,7 @@ static const char rcsid[] = "$Id: connection.c,v 1.28 2001/10/18 10:10:50 chris
 extern int verbose;
 
 /* make_timestamp:
- * Create a timestamp string.
- */
+ * Create a timestamp string. */
 #define TIMESTAMP_LEN   32
 static char hex[] = "0123456789abcdef";
 
@@ -89,8 +88,7 @@ static char *make_timestamp(const char *domain) {
 }
 
 /* connection_new:
- * Create a connection object from a socket.
- */
+ * Create a connection object from a socket. */
 connection connection_new(int s, const struct sockaddr_in *sin, const char *domain) {
     connection c = 0;
 
@@ -99,7 +97,7 @@ connection connection_new(int s, const struct sockaddr_in *sin, const char *doma
     c->s = s;
     memcpy(&(c->sin), sin, sizeof(struct sockaddr_in));
 
-    if (domain) c->domain = strdup(domain);
+    if (domain) c->domain = xstrdup(domain);
 
     c->idstr = xmalloc(strlen(inet_ntoa(sin->sin_addr)) + 1 + (domain ? strlen(domain) : 0) + 16);
     if (domain) sprintf(c->idstr, "[%d]%s/%s", s, inet_ntoa(sin->sin_addr), domain);
@@ -116,7 +114,7 @@ connection connection_new(int s, const struct sockaddr_in *sin, const char *doma
     c->idlesince = time(NULL);
 
     if (!connection_sendresponse(c, 1, c->timestamp)) {
-        print_log(LOG_ERR, "connection_new: could not send timestamp to `%s'", c->idstr);
+        log_print(LOG_ERR, "connection_new: could not send timestamp to `%s'", c->idstr);
         goto fail;
     }
 
@@ -128,8 +126,7 @@ connection connection_new(int s, const struct sockaddr_in *sin, const char *doma
 }
 
 /* connection_delete:
- * Delete a connection and disconnect the peer.
- */
+ * Delete a connection and disconnect the peer. */
 void connection_delete(connection c) {
     if (!c) return;
 
@@ -152,13 +149,12 @@ void connection_delete(connection c) {
 
 /* connection_read:
  * Read data from the socket into the buffer, if available. Returns -1 on
- * error or buffer full, 0 on EOF or the number of bytes read.
- */
+ * error or buffer full, 0 on EOF or the number of bytes read. */
 ssize_t connection_read(connection c) {
     ssize_t n;
     if (!c) return -1;
     if (c->p == c->buffer + c->bufferlen) {
-        print_log(LOG_ERR, _("connection_read: client %s: over-long line"), c->idstr);
+        log_print(LOG_ERR, _("connection_read: client %s: over-long line"), c->idstr);
         errno = ENOBUFS;
         return -1;
     }
@@ -187,8 +183,7 @@ static void dump(const char *s, size_t l) {
 */
 
 /* connection_parsecommand:
- * Parse a command from the connection, returning NULL if none is available.
- */
+ * Parse a command from the connection, returning NULL if none is available. */
 struct {
     char *s;
     enum pop3_command_code cmd;
@@ -238,7 +233,7 @@ pop3command connection_parsecommand(connection c) {
             if (i != pc->toks->num - 1) strcat(s, " ");
         }
         strcat(s, "'");
-        print_log(LOG_DEBUG, "%s", s);
+        log_print(LOG_DEBUG, "%s", s);
         xfree(s);
     }
                 
@@ -250,8 +245,7 @@ pop3command connection_parsecommand(connection c) {
 }
 
 /* pop3command_new:
- * Create a new pop3command object.
- */
+ * Create a new pop3command object. */
 pop3command pop3command_new(const char *s) {
     pop3command p;
     int i;
@@ -276,8 +270,7 @@ pop3command pop3command_new(const char *s) {
 }
 
 /* pop3command_delete:
- * Free a command returned by pop3command_new.
- */
+ * Free a command returned by pop3command_new. */
 void pop3command_delete(pop3command p) {
     if (!p) return;
     if (p->toks) tokens_delete(p->toks);
@@ -286,8 +279,7 @@ void pop3command_delete(pop3command p) {
 
 /* connection_sendresponse:
  * Send a +OK... / -ERR... response to a message. Returns 1 on success or 0 on
- * failure.
- */
+ * failure. */
 int connection_sendresponse(connection c, const int success, const char *s) {
     char *x;
     size_t l, m;
@@ -297,14 +289,13 @@ int connection_sendresponse(connection c, const int success, const char *s) {
     m = xwrite(c->s, x, l = strlen(x));
     xfree(x);
     if (verbose)
-        print_log(LOG_DEBUG, _("connection_sendresponse: client %s: sent `%s %s'"), c->idstr, success? "+OK" : "-ERR", s);
+        log_print(LOG_DEBUG, _("connection_sendresponse: client %s: sent `%s %s'"), c->idstr, success? "+OK" : "-ERR", s);
     return (m == l);
 }
 
 /* connection_sendline:
  * Send an arbitrary line to a connected peer. Returns 1 on success or 0 on
- * failure. Used to send multiline responses.
- */
+ * failure. Used to send multiline responses. */
 int connection_sendline(connection c, const char *s) {
     char *x;
     size_t l, m;
