@@ -13,30 +13,30 @@
 
 #ifdef MBOX_MAILDIR
 
-static const char rcsid[] = "$Id: maildir.c,v 1.4 2001/10/14 01:13:38 chris Exp $";
+static const char rcsid[] = "$Id: maildir.c,v 1.9 2002/04/03 20:07:41 chris Exp $";
 
 #include <sys/types.h>     /* u_int_* for dirent.h */
 #include <dirent.h>        /* DIR, etc */
 #include <syslog.h>        /* LOG_* */
 #include <unistd.h>        /* chdir() */
-#include <string.h>        /* strdup() */
+#include <string.h>
 #include <stdlib.h>
 #include <sys/fcntl.h>     /* O_RDONLY */
 #include <sys/time.h>
 #include <stdio.h>         /* rename() */
 #include <errno.h>
+#include <unistd.h>
 
 #include "mailbox.h"
 #include "util.h"
 #include "vector.h"
 
 /* maildir_make_indexpoint:
- * Make an indexpoint to put in a maildir.
- */
+ * Make an indexpoint to put in a maildir. */
 static void maildir_make_indexpoint(struct indexpoint *m, const char *filename, off_t size, time_t mtime) {
     memset(m, 0, sizeof(struct indexpoint));
 
-    m->filename = strdup(filename);
+    m->filename = xstrdup(filename);
     if (!m->filename) {
         return;
     }
@@ -44,8 +44,16 @@ static void maildir_make_indexpoint(struct indexpoint *m, const char *filename,
     m->length = 0;    /* "\n\nFrom " delimiter not used */
     m->deleted = 0;
     m->msglength = size;
-    /* XXX this could break uniqueness, though in practice this is unlikely. */
-    strncpy(m->hash, filename+4, sizeof(m->hash));    /* +4: skip cur/ or new/ subdir */
+
+    /* In previous versions of tpop3d, the first 16 characters of the file name
+     * of a maildir message were used to form a unique ID. Unfortunately, this
+     * is not a good strategy, especially now that time_t's are 10 characters
+     * long. So now we form an MD5 hash of the file name; obviously, these
+     * unique IDs are not compatible with the old ones, so optionally you can
+     * retain the old scheme by replacing the following line with
+     *     strncpy(m->hash, filename+4, sizeof(m->hash));
+     */
+    md5_digest(filename + 4, strcspn(filename + 4, ":"), m->hash); /* +4: skip cur/ or new/ subdir; ignore flags at end. */
     
     m->mtime = mtime;
 }
@@ -53,8 +61,7 @@ static void maildir_make_indexpoint(struct indexpoint *m, const char *filename,
 /* maildir_build_index:
  * Build an index of a maildir. subdir is one of cur, tmp or new; time is the
  * time at which the operation started, used to ignore messages delivered
- * during processes. Returns 0 on success, -1 otherwise.
- */
+ * during processes. Returns 0 on success, -1 otherwise. */
 int maildir_build_index(mailbox M, const char *subdir, time_t time) {
     DIR *dir;
     struct dirent *d;
@@ -63,7 +70,7 @@ int maildir_build_index(mailbox M, const char *subdir, time_t time) {
 
     dir = opendir(subdir);
     if (!dir) {
-        print_log(LOG_ERR, "maildir_build_index: opendir(%s): %m", subdir);
+        log_print(LOG_ERR, "maildir_build_index: opendir(%s): %m", subdir);
         return -1;
     }
     
@@ -80,15 +87,15 @@ int maildir_build_index(mailbox M, const char *subdir, time_t time) {
             struct indexpoint pt;
             maildir_make_indexpoint(&pt, filename, st.st_size, st.st_mtime);
             mailbox_add_indexpoint(M, &pt);
-            /* Hack: accumulate size of messages. */
-            M->st.st_size += st.st_size;
+            /* Accumulate size of messages. */
+            M->totalsize += st.st_size;
         }
         xfree(filename);
     }
     closedir(dir);
     
     if (d) {
-        print_log(LOG_ERR, "maildir_build_index: readdir(%s): %m", subdir);
+        log_print(LOG_ERR, "maildir_build_index: readdir(%s): %m", subdir);
         return -1;
     }
 
@@ -100,16 +107,14 @@ int maildir_build_index(mailbox M, const char *subdir, time_t time) {
 }
 
 /* maildir_sort_callback:
- * qsort(3) callback for ordering messages in a maildir.
- */
+ * qsort(3) callback for ordering messages in a maildir. */
 int maildir_sort_callback(const void *a, const void *b) {
     const struct indexpoint *A = a, *B = b;
     return A->mtime - B->mtime;
 }
 
 /* maildir_new:
- * Create a mailbox object from a maildir.
- */
+ * Create a mailbox object from a maildir. */
 mailbox maildir_new(const char *dirname) {
     mailbox M, failM = NULL;
     struct timeval tv1, tv2;
@@ -128,14 +133,10 @@ mailbox maildir_new(const char *dirname) {
     
     if (chdir(dirname) == -1) {
         if (errno == ENOENT) failM = MBOX_NOENT;
-        else print_log(LOG_ERR, "maildir_new: chdir(%s): %m", dirname);
+        else log_print(LOG_ERR, "maildir_new: chdir(%s): %m", dirname);
         goto fail;
-    } else {
-        if(!(M->name = strdup(dirname))) {
-            print_log(LOG_ERR, "maildir_new: strdup: %m");
-            goto fail;
-        }
-    }
+    } else
+        M->name = xstrdup(dirname);
     
     gettimeofday(&tv1, NULL);
     
@@ -148,7 +149,7 @@ mailbox maildir_new(const char *dirname) {
 
     gettimeofday(&tv2, NULL);
     f = (float)(tv2.tv_sec - tv1.tv_sec) + 1e-6 * (float)(tv2.tv_usec - tv1.tv_usec);
-    print_log(LOG_DEBUG, "maildir_new: scanned maildir %s (%d messages) in %0.3fs", dirname, (int)M->num, f);
+    log_print(LOG_DEBUG, "maildir_new: scanned maildir %s (%d messages) in %0.3fs", dirname, (int)M->num, f);
     
     return M;
 
@@ -165,8 +166,7 @@ fail:
  * maildir, escaping lines which begin . as required by RFC1939. Returns 1
  * on success or 0 on failure. The whole message is sent if n == -1.
  *
- * XXX Assumes that maildirs use only '\n' to indicate EOL.
- */
+ * XXX Assumes that maildirs use only '\n' to indicate EOL. */
 int maildir_send_message(const mailbox M, int sck, const int i, int n) {
     struct indexpoint *m;
     int fd, status;
@@ -176,10 +176,10 @@ int maildir_send_message(const mailbox M, int sck, const int i, int n) {
     m = M->index +i;
     fd = open(m->filename, O_RDONLY);
     if (fd == -1) {
-        print_log(LOG_ERR, "maildir_send_message: open(%s): %m", m->filename);
+        log_print(LOG_ERR, "maildir_send_message: open(%s): %m", m->filename);
         return 0;
     }
-    print_log(LOG_INFO, "maildir_send_message: sending message %d (%s) size %d bytes", i+1, m->filename, m->msglength);
+    log_print(LOG_INFO, "maildir_send_message: sending message %d (%s) size %d bytes", i+1, m->filename, m->msglength);
     status = write_file(fd, sck, 0 /* offset */, 0 /* skip */, m->msglength, n);
     close(fd);
 
@@ -187,25 +187,22 @@ int maildir_send_message(const mailbox M, int sck, const int i, int n) {
 }
 
 /* maildir_apply_changes:
- * Apply deletions to a maildir.
- */
+ * Apply deletions to a maildir. */
 int maildir_apply_changes(mailbox M) {
     struct indexpoint *m;
     if (!M) return 1;
 
     for (m = M->index; m < M->index + M->num; ++m) {
         if (m->deleted) {
-            if (unlink(m->filename) == -1) {
-                print_log(LOG_ERR, "maildir_apply_changes: unlink(%s): %m", m->filename);
-                return 0;
-            }
+            if (unlink(m->filename) == -1)
+                log_print(LOG_ERR, "maildir_apply_changes: unlink(%s): %m", m->filename);
+                /* Warn but proceed anyway. */
         } else {
             /* Mark message read. */
             if (strncmp(m->filename, "new/", 4) == 0) {
                 char *cur;
-                cur = xmalloc(6 + strlen(m->filename) - 4);
-                sprintf(cur, "cur/%s", m->filename + 4);
-                if (!cur) return 0;
+                cur = xmalloc(strlen(m->filename) + 5);
+                sprintf(cur, "cur/%s:2,S", m->filename + 4); /* Set seen flag */
                 rename(m->filename, cur);    /* doesn't matter if it can't */
                 xfree(cur);
             }
