@@ -26,7 +26,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef MBOX_BSD
-static const char rcsid[] = "$Id: mailspool.c,v 1.31 2001/10/14 01:13:38 chris Exp $";
+static const char rcsid[] = "$Id: mailspool.c,v 1.37 2002/03/18 23:47:33 chris Exp $";
 
 #include <errno.h>
 #include <fcntl.h>
@@ -61,8 +61,7 @@ int mailspool_load_index(mailbox m);
 #endif /* MBOX_BSD_SAVE_INDICES */
 
 /* file_unlock:
- * Unlock a mailspool file. Returns 1 on success or 0 on failure.
- */
+ * Unlock a mailspool file. Returns 1 on success or 0 on failure. */
 int file_unlock(const int fd, const char *name) {
     int r = 1;
  #ifdef WITH_FCNTL_LOCKING
@@ -83,8 +82,7 @@ int file_unlock(const int fd, const char *name) {
 /* file_lock:
  * Lock a mailspool file. Returns 1 on success or 0 on failure. This uses
  * whatever locking strategies the user has selected with compile-time
- * definitions.
- */
+ * definitions. */
 int file_lock(const int fd, const char *name) {
     int l_fcntl, l_flock, l_dotfile;
     l_fcntl = l_flock = l_dotfile = 0;
@@ -122,8 +120,7 @@ fail:
 }
 
 /* mailspool_make_indexpoint:
- * Make an indexpoint.
- */
+ * Make an indexpoint. */
 void mailspool_make_indexpoint(struct indexpoint *x, const size_t offset, const size_t length, const size_t msglength, const unsigned char *hash) {
     memset(x, 0, sizeof(struct indexpoint));
 
@@ -134,8 +131,7 @@ void mailspool_make_indexpoint(struct indexpoint *x, const size_t offset, const
 }
 
 /* mailspool_new_from_file:
- * Open a file, lock it, and form an index of the messages in it.
- */
+ * Open a file, lock it, and form an index of the messages in it. */
 mailbox mailspool_new_from_file(const char *filename) {
     mailbox M, failM = NULL;
     int i;
@@ -155,18 +151,17 @@ mailbox mailspool_new_from_file(const char *filename) {
     
     if (stat(filename, &(M->st)) == -1) {
         /* If the mailspool doesn't exist, fail silently, since this may be
-         * getting called from find_mailbox.
-         */
+         * getting called from find_mailbox. */
         if (errno == ENOENT) failM = MBOX_NOENT;
-        else print_log(LOG_INFO, "mailspool_new_from_file: stat(%s): %m", filename);
+        else log_print(LOG_INFO, "mailspool_new_from_file: stat(%s): %m", filename);
         goto fail;
-    } else M->name = strdup(filename);
+    } else M->name = xstrdup(filename);
     
     /* Naive locking strategy. */
     for (i = 0; i < MAILSPOOL_LOCK_TRIES; ++i) {
         M->fd = open(M->name, O_RDWR);
         if (M->fd == -1) {
-            print_log(LOG_ERR, "mailspool_new_from_file: %m");
+            log_print(LOG_ERR, "mailspool_new_from_file: %m");
             goto fail;
         }
      
@@ -179,7 +174,7 @@ mailbox mailspool_new_from_file(const char *filename) {
     }
 
     if (M->fd == -1) {
-        print_log(LOG_ERR, _("mailspool_new_from_file: failed to lock %s: %m"), filename);
+        log_print(LOG_ERR, _("mailspool_new_from_file: failed to lock %s: %m"), filename);
         goto fail;
     }
 
@@ -189,19 +184,23 @@ mailbox mailspool_new_from_file(const char *filename) {
 #ifdef MBOX_BSD_SAVE_INDICES
     if (mailspool_save_indices) {
         if (mailspool_load_index(M) == -1) {
-            print_log(LOG_ERR, _("mailspool_new_from_file: unable to index mailspool"));
+            log_print(LOG_ERR, _("mailspool_new_from_file: unable to index mailspool"));
             goto fail;
         }
     } else
 #endif
     if (mailspool_build_index(M, NULL) == -1) {
-        print_log(LOG_ERR, _("mailspool_new_from_file: unable to index mailspool"));
+        log_print(LOG_ERR, _("mailspool_new_from_file: unable to index mailspool"));
         goto fail;
     }
 
     gettimeofday(&tv2, NULL);
     f = (float)(tv2.tv_sec - tv1.tv_sec) + 1e-6 * (float)(tv2.tv_usec - tv1.tv_usec);
-    print_log(LOG_DEBUG, _("mailspool_new_from_file: indexed mailspool %s (%d bytes) in %0.3fs"), filename, (int)M->st.st_size, f);
+    log_print(LOG_DEBUG, _("mailspool_new_from_file: indexed mailspool %s (%d bytes) in %0.3fs"), filename, (int)M->st.st_size, f);
+    
+    /* OK, now go through the mailspool and accumulate statistics. */
+    for (i = 0, M->totalsize = 0; i < M->num; ++i)
+        M->totalsize += M->index[i].msglength;
     
     return M;
 
@@ -211,8 +210,7 @@ fail:
 }
 
 /* mailspool_delete:
- * Deletion specific to mailspools.
- */
+ * Deletion specific to mailspools. */
 void mailspool_delete(mailbox m) {
     if (!m) return;
     if (m->name) file_unlock(m->fd, m->name);
@@ -223,36 +221,29 @@ void mailspool_delete(mailbox m) {
 
 /* memstr:
  * Locate needle, of length n_len, in haystack, of length h_len, returning
- * NULL if it is not found.
- */
+ * NULL if it is not found. */
 static char *memstr(const char *haystack, size_t h_len, const char *needle, size_t n_len)
 {
     const char *p;
 
     if (n_len > h_len)
-    return NULL;
+        return NULL;
 
-    p = (const char*) memchr(haystack, *needle, h_len - n_len);
+    p = (const char*)memchr(haystack, *needle, h_len - n_len + 1);
     while (p) {
-    if (!memcmp(p, needle, n_len))
-        return (char*)p;
-    else
-        p = (const char*)memchr(p + 1, *needle, (haystack + h_len - n_len) - p - 1);
+        if (!memcmp(p, needle, n_len))
+            return (char*)p;
+        else
+            p = (const char*)memchr(p + 1, *needle, h_len - n_len - (p - haystack));
     }
 
     return NULL;
 }
 
-/*
-struct timeval TT;
-#define ts(a)  do { struct timeval tv; float t; gettimeofday(&tv, NULL); t = (double)(tv.tv_sec - TT.tv_sec) + 1e-6 * (double)(tv.tv_usec - TT.tv_usec); print_log(LOG_DEBUG, a " delta = %lf", t); TT = tv; } while (0)
-*/
-
 /* mailspool_build_index:
  * Build an index of a mailspool. Uses mmap(2) for speed. Assumes that
  * mailspools use only '\n' to indicate EOL. Returns 0 on success or -1 on
- * failure.
- */
+ * failure. */
 int mailspool_build_index(mailbox M, char *filemem) {
     char *p, *q;
     size_t len, len2;
@@ -268,19 +259,18 @@ int mailspool_build_index(mailbox M, char *filemem) {
     if (!filemem) {
         filemem = mmap(0, len, PROT_READ, MAP_PRIVATE, M->fd, 0);
         if (filemem == MAP_FAILED) {
-            print_log(LOG_ERR, "mailspool_build_index(%s): mmap: %m", M->name);
+            log_print(LOG_ERR, "mailspool_build_index(%s): mmap: %m", M->name);
             return -1;
         }
     }
 
     if (M->num > 0) {
         /* Perhaps we are parsing the tail of the file, after reading a
-         * partial index?
-         */
+         * partial index? */
         struct indexpoint *P = M->index + M->num - 1;
         p = filemem + P->offset + P->msglength - 2;
         first = M->num;
-        print_log(LOG_DEBUG, _("mailspool_build_index(%s): first %d messages indexed from cached metadata"), M->name, first);
+        log_print(LOG_DEBUG, _("mailspool_build_index(%s): first %d messages indexed from cached metadata"), M->name, first);
     } else
         /* Nope, never seen this one before. */
         p = filemem - 2;
@@ -313,18 +303,12 @@ int mailspool_build_index(mailbox M, char *filemem) {
         t->msglength = M->st.st_size - t->offset;
 
         /* We generate "unique" IDs by hashing the first 512 or so bytes of the
-         * data in each message. Only do this for messages we've just found.
-         */
+         * data in each message. Only do this for newly found messages. */
         for (t = M->index; t < M->index + M->num; ++t) {
-            MD5_CTX ctx;
             size_t n = 512;
 
             if (n > t->msglength) n = t->msglength;
-            
-            /* Compute MD5 */
-            MD5Init(&ctx);
-            MD5Update(&ctx, (unsigned char*)filemem + t->offset, n);
-            MD5Final(t->hash, &ctx);
+            md5_digest((void*)(filemem + t->offset), n, t->hash);
         }
     }
 
@@ -336,15 +320,14 @@ int mailspool_build_index(mailbox M, char *filemem) {
      *  Subject: DON'T DELETE THIS MESSAGE -- FOLDER INTERNAL DATA
      *  X-IMAP: <some numbers>
      *
-     * The metadata message is assumed to be the first one in the mailspool.
-     */
+     * The metadata message is assumed to be the first one in the mailspool. */
     if (M->num >= 1) {
         struct indexpoint *P = M->index;
         p = memstr(filemem + P->offset, P->msglength, "\n\n", 2);
         if (p) {
             const char hdr1[] = "\nX-IMAP: ", hdr2[] = "Subject: DON'T DELETE THIS MESSAGE -- FOLDER INTERNAL DATA\n";
             if (memstr(filemem + P->offset, p - filemem, hdr1, strlen(hdr1)) && memstr(filemem + P->offset, p - filemem, hdr2, strlen(hdr2))) {
-                print_log(LOG_DEBUG, "mailspool_build_index(%s): skipping c-client metadata", M->name);
+                log_print(LOG_DEBUG, "mailspool_build_index(%s): skipping c-client metadata", M->name);
                 memmove((void*)M->index, (void*)(M->index + 1), sizeof(struct indexpoint) * (M->num - 1));
                 --M->num;
             }
@@ -353,13 +336,11 @@ int mailspool_build_index(mailbox M, char *filemem) {
 #endif /* IGNORE_CCLIENT_METADATA */
 
     munmap(filemem, len);
-/* ts("munmap");     */
     return 0;
 }
 
 /* mailspool_send_message:
- * Front-end to write_file in util.c.
- */
+ * Front-end to write_file in util.c. */
 int mailspool_send_message(const mailbox M, int sck, const int i, int n) {
     struct indexpoint *x;
 
@@ -415,8 +396,7 @@ int mailspool_send_message(const mailbox M, int sck, const int i, int n) {
  * d + (K->offset - J->offset), to take account of the hole we made.
  * 
  * A special case occurs where the section to be deleted is at the end of the
- * file, at which point we can just ftruncate(2).
- */
+ * file, at which point we can just ftruncate(2). */
 int mailspool_apply_changes(mailbox M) {
     char *filemem, *s, *d;
     size_t len;
@@ -432,7 +412,7 @@ int mailspool_apply_changes(mailbox M) {
          * first message.
          */
         if (ftruncate(M->fd, M->index->offset) == -1) {
-            print_log(LOG_ERR, "mailspool_apply_changes(%s): ftruncate: %m", M->name);
+            log_print(LOG_ERR, "mailspool_apply_changes(%s): ftruncate: %m", M->name);
             return 0;
         } else return 1;
     }
@@ -442,7 +422,7 @@ int mailspool_apply_changes(mailbox M) {
     len += PAGESIZE - (len % PAGESIZE);
     filemem = mmap(0, len, PROT_WRITE | PROT_READ, MAP_SHARED, M->fd, 0);
     if (filemem == MAP_FAILED) {
-        print_log(LOG_ERR, "mailspool_apply_changes(%s): mmap: %m", M->name);
+        log_print(LOG_ERR, "mailspool_apply_changes(%s): mmap: %m", M->name);
         close(M->fd);
         return 0;
     }
@@ -454,8 +434,8 @@ int mailspool_apply_changes(mailbox M) {
     /* Find the first message to be deleted. */
     while (I < End && !I->deleted) ++I;
     if (I == End) {
-        if (munmap(filemem, len) == -1) print_log(LOG_ERR, "mailspool_send_message: munmap: %m");
-        print_log(LOG_ERR, _("mailspool_apply_changes(%s): inconsistency in mailspool data"), M->name);
+        if (munmap(filemem, len) == -1) log_print(LOG_ERR, "mailspool_send_message: munmap: %m");
+        log_print(LOG_ERR, _("mailspool_apply_changes(%s): inconsistency in mailspool data"), M->name);
         return 0;
     }
     d = filemem + I->offset;
@@ -473,8 +453,7 @@ int mailspool_apply_changes(mailbox M) {
             while (K < End && !K->deleted) copylen += (K++)->msglength;
 
             /* Not every machine has a working memmove(3) (allows overlapping
-             * memory areas). If yours doesn't, you should get a better one ;)
-             */
+             * memory areas). If yours doesn't, get a better one ;) */
             memmove(d, s, copylen);
             d += copylen;
         }
@@ -484,20 +463,20 @@ int mailspool_apply_changes(mailbox M) {
 
     /* Truncate the very end. */
     if (ftruncate(M->fd, d - filemem) == -1) {
-        print_log(LOG_ERR, "mailspool_apply_changes(%s): ftruncate: %m", M->name);
-        if (munmap(filemem, len) == -1) print_log(LOG_ERR, "mailspool_send_message: munmap: %m");
+        log_print(LOG_ERR, "mailspool_apply_changes(%s): ftruncate: %m", M->name);
+        if (munmap(filemem, len) == -1) log_print(LOG_ERR, "mailspool_send_message: munmap: %m");
         return 0;
     }
     
     /* Done, unmap the file. */
     if (munmap(filemem, len) == -1) {
-        print_log(LOG_ERR, "mailspool_send_message: munmap: %m");
+        log_print(LOG_ERR, "mailspool_send_message: munmap: %m");
         return 0;
     }
 
 #ifdef MBOX_BSD_SAVE_INDICES
     if (mailspool_save_indices && !mailspool_save_index(M))
-        print_log(LOG_WARNING, _("mailspool_apply_changes(%s): unable to save mailspool index"), M->name);
+        log_print(LOG_WARNING, _("mailspool_apply_changes(%s): unable to save mailspool index"), M->name);
 #endif /* MBOX_BSD_SAVE_INDICES */
     
     return 1;
@@ -523,13 +502,11 @@ int mailspool_apply_changes(mailbox M) {
  * some dumb pieces of software (I love you, PINE!) will modify them for some
  * deranged reason probably known only to people who live up in that corner of
  * the world among the giant redwoods, collapsing suspension bridges, and
- * 1970s software empires.
- */
+ * 1970s software empires. */
 
 /* mailspool_find_index:
  * Find the name of a mailspool's index file, using the spec in the config
- * file.
- */
+ * file. */
 extern stringmap config;
 
 char *mailspool_find_index(mailbox m) {
@@ -559,13 +536,11 @@ char *mailspool_find_index(mailbox m) {
      *
      * In either case, the path in which the index is saved needs to have
      * permissions which allow the user who owns the mailspool to write a new
-     * file to it. 1777 would be traditional.
-     *
-     */
-    path = strdup(m->name);
+     * file to it. 1777 would be traditional. */
+    path = xstrdup(m->name);
     p = strrchr(path, '/');
     if (p) *p = 0;
-    file = strdup(p + 1);
+    file = xstrdup(p + 1);
     escaped_name = xcalloc(strlen(m->name) * 3 + 2, 1);
 
     /* Form HTTP-style escaped version of name. Only escape % and /, though. */
@@ -578,7 +553,7 @@ char *mailspool_find_index(mailbox m) {
 
     indexname = substitute_variables(subspec, &err, 4, "name", m->name, "path", path, "file", file, "escaped_name", escaped_name);
     if (!indexname) {
-        print_log(LOG_ERR, _("mailspool_find_index: %s near `%.16s'"), err.msg, subspec + err.offset);
+        log_print(LOG_ERR, _("mailspool_find_index: %s near `%.16s'"), err.msg, subspec + err.offset);
         goto fail;
     }
 
@@ -601,8 +576,7 @@ char index_signature[] =
  * Save an index of a mailspool. Returns 1 on success or 0 on failure. Uses
  * stdio, which is unfortunate but makes it a bit easier to write. The
  * mailspool must be locked when this is called. Makes some attempt to avoid
- * clobbering files via symlink attacks.
- */
+ * clobbering files via symlink attacks. */
 int mailspool_save_index(mailbox m) {
     char *indexfile;
     int ret = 0;
@@ -621,19 +595,18 @@ int mailspool_save_index(mailbox m) {
     /* OK, now we need to save the thing. */
     fd = open(indexfile, O_RDWR | O_CREAT, 0600); /* Ensure correct permissions. */
     if (fd == -1) {
-        print_log(LOG_ERR, "mailspool_save_index(%s): %m", indexfile);
+        log_print(LOG_ERR, "mailspool_save_index(%s): %m", indexfile);
         goto fail;
     }
 
     /* Now we need to make sure that there isn't some sort of childish symlink
-     * attack in progress.
-     */
+     * attack in progress. */
     a = readlink(indexfile, buf, sizeof(buf));
     if (a == 0) {
-        print_log(LOG_ERR, _("mailspool_save_index(%s): possible security problem: index file exists and is a symlink to `%s'"), indexfile, buf);
+        log_print(LOG_ERR, _("mailspool_save_index(%s): possible security problem: index file exists and is a symlink to `%s'"), indexfile, buf);
         goto fail;
     } else if (a == -1 && errno != EINVAL) {
-        print_log(LOG_ERR, _("mailspool_save_index(%s): possible security problem: index file exists, and is a symlink, but readlink failed: %m"), indexfile);
+        log_print(LOG_ERR, _("mailspool_save_index(%s): possible security problem: index file exists, and is a symlink, but readlink failed: %m"), indexfile);
         goto fail;
     }
 
@@ -642,7 +615,7 @@ int mailspool_save_index(mailbox m) {
     
     fp = fdopen(fd, "wt");
     if (!fp) {
-        print_log(LOG_ERR, "mailspool_save_index(%s): %m", indexfile);
+        log_print(LOG_ERR, "mailspool_save_index(%s): %m", indexfile);
         goto fail;
     }
 
@@ -681,10 +654,9 @@ fail:
  * Attempts to construct a mailspool index from a saved index file, if one
  * exists and is of the correct format. We may find that we need to re-parse
  * the tail of the file; this is done by calling into the `normal'
- * mailspool_build_index. Returns 0 on success or -1 on failure.
- */
+ * mailspool_build_index. Returns 0 on success or -1 on failure. */
 int mailspool_load_index(mailbox m) {
-    char *indexfile;
+    char *indexfile = NULL;
     FILE *fp = NULL;
     struct stat st;
     int offset, length, msglength;
@@ -701,32 +673,30 @@ int mailspool_load_index(mailbox m) {
 
     fp = fopen(indexfile, "rt");
     if (!fp) {
-        print_log(LOG_WARNING, "mailspool_load_index(%s): %m", indexfile);
+        log_print(LOG_WARNING, "mailspool_load_index(%s): %m", indexfile);
         goto fail;
     }
 
     /* Security. The file must have the correct permissions, and be owned by
-     * ourselves.
-     */
+     * ourselves. */
     if (fstat(fileno(fp), &st) == -1) {
-        print_log(LOG_ERR, "mailspool_load_index(%s): %m", indexfile);
+        log_print(LOG_ERR, "mailspool_load_index(%s): %m", indexfile);
         goto fail;
     } else if ((st.st_mode & 0777) != 0600 || m->st.st_uid != getuid()) {
-        print_log(LOG_ERR, _("mailspool_load_index(%s): possible security problem: index exists, but it has the wrong owner or file permissions"), indexfile);
-        print_log(LOG_ERR, _("mailspool_load_index(%s): owner is %d, should be %d; mode 0%o, should be 0600"), indexfile,
+        log_print(LOG_ERR, _("mailspool_load_index(%s): possible security problem: index exists, but it has the wrong owner or file permissions"), indexfile);
+        log_print(LOG_ERR, _("mailspool_load_index(%s): owner is %d, should be %d; mode 0%o, should be 0600"), indexfile,
                             m->st.st_uid, getuid(), m->st.st_mode & 0777);
         goto fail;
     }
 
     /* OK, found an index file; let's try loading some data out of it. */
     if (fread(sigbuf, 1, sizeof(sigbuf) - 1, fp) != sizeof(sigbuf) - 1 || memcmp(sigbuf, index_signature, sizeof(sigbuf) - 1) != 0) {
-        print_log(LOG_WARNING, _("mailspool_load_index(%s): index exists, but is of wrong format; ignoring"), indexfile);
+        log_print(LOG_WARNING, _("mailspool_load_index(%s): index exists, but is of wrong format; ignoring"), indexfile);
         goto fail;
     }
 
     /* Should now get a bunch of offset/hash lines. Stuff these into the
-     * mailbox object. Also mmap the real mailspool so we can check these.
-     */
+     * mailbox object. Also mmap the real mailspool so we can check these. */
     len = len2 = m->st.st_size;
 
     if (len < 16) goto fail;
@@ -734,13 +704,12 @@ int mailspool_load_index(mailbox m) {
     len += PAGESIZE - (len % PAGESIZE);
     filemem = mmap(0, len, PROT_READ, MAP_PRIVATE, m->fd, 0);
     if (filemem == MAP_FAILED) {
-        print_log(LOG_ERR, "mailspool_load_index(%s): mmap: %m", m->name);
+        log_print(LOG_ERR, "mailspool_load_index(%s): mmap: %m", m->name);
         goto fail;
     }
 
     while (fscanf(fp, "%8x %8x %8x %32[0-9a-f]", &offset, &length, &msglength, hexdigest) == 4) {
         struct indexpoint x;
-        MD5_CTX ctx;
         size_t n = 512;
         unsigned char realhash[16];
 
@@ -754,19 +723,23 @@ int mailspool_load_index(mailbox m) {
         if (n > x.msglength) n = x.msglength;
 
         /* Compute MD5 */
-        MD5Init(&ctx);
-        MD5Update(&ctx, (unsigned char*)filemem + x.offset, n);
-        MD5Final(realhash, &ctx);
+        md5_digest(filemem + x.offset, n, realhash);
 
-        if (memcmp(realhash, x.hash, 16) != 0)
+        /* No match; stop. */
+        if (memcmp(realhash, x.hash, 16) != 0) {
+            /* Get rid of any preceding record: we will have to re-index that
+             * one, too. */
+            if (m->num > 0)
+                --m->num;
             break;
+        }
 
         /* OK, this message seems to have been indexed correctly.... */
         mailbox_add_indexpoint(m, &x);
     }
 
     if (!feof(fp)) {
-        print_log(LOG_WARNING, _("mailspool_load_index(%s): index exists, but has some stale or corrupt data"), indexfile);
+        log_print(LOG_WARNING, _("mailspool_load_index(%s): index exists, but has some stale or corrupt data"), indexfile);
         goto fail;
     }
 
