@@ -6,7 +6,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: pop3.c,v 1.34 2001/10/14 01:13:38 chris Exp $";
+static const char rcsid[] = "$Id: pop3.c,v 1.40 2002/03/18 23:47:33 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -31,9 +31,9 @@ extern int verbose;
 
 /* connection_do:
  * Makes a command do something on a connection, returning a code indicating
- * what the caller should do.
- */
+ * what the caller should do. */
 int append_domain;  /* Do we automatically try user@domain if user alone fails to authenticate? */
+int strip_domain; /* Automatically try user if user@domain fails? */
 
 enum connection_action connection_do(connection c, const pop3command p) {
     /* This breaks the RFC, but is sensible. */
@@ -50,7 +50,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 connection_sendresponse(c, 0, _("But you already said `USER'."));
                 return do_nothing;
             } else {
-                c->user = strdup((char*)p->toks->toks[1]);
+                c->user = xstrdup((char*)p->toks->toks[1]);
                 if (!c->user) {
 #ifndef NO_SNIDE_COMMENTS
                     connection_sendresponse(c, 0, _("Tell me your name, knave!"));
@@ -70,7 +70,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 connection_sendresponse(c, 0, _("But you already said `PASS'."));
                 return do_nothing;
             } else {
-                c->pass = strdup(p->toks->toks[1]);
+                c->pass = xstrdup(p->toks->toks[1]);
                 if (!c->pass) {
                     connection_sendresponse(c, 0, _("You must give a password."));
                     return do_nothing;
@@ -125,16 +125,25 @@ enum connection_action connection_do(connection c, const pop3command p) {
                     return do_nothing;
                 }
 
-                c->a = authcontext_new_apop(name, c->timestamp, digest, c->domain, inet_ntoa(c->sin.sin_addr));
-                
-                if (!c->a && append_domain && c->domain && strcspn(name, "@%!") == strlen(name)) {
-                    /* OK, if we have a domain name, try appending that. */
-                    char *nn = xmalloc(strlen(name) + strlen(c->domain) + 2);
-                    strcpy(nn, name);
-                    strcat(nn, "@");
-                    strcat(nn, c->domain);
-                    c->a = authcontext_new_apop(nn, c->timestamp, digest, c->domain, inet_ntoa(c->sin.sin_addr));
-                    xfree(nn);
+                c->a = authcontext_new_apop(name, NULL, c->domain, c->timestamp, digest, inet_ntoa(c->sin.sin_addr));
+ 
+                /* Maybe retry authentication with an added or removed domain
+                 * name. */
+                if (!c->a && (strip_domain || append_domain)) {
+                    int n, len;
+                    len = strlen(name);
+                    n = strcspn(name, DOMAIN_SEPARATORS);
+                    if (append_domain && c->domain && n == len)
+                        /* OK, if we have a domain name, try appending that. */
+                        c->a = authcontext_new_apop(name, name, c->domain, c->timestamp, digest, inet_ntoa(c->sin.sin_addr));
+                    else if (strip_domain && n != len) {
+                        /* Try stripping off the supplied domain name. */
+                        char *u;
+                        u = xstrdup(name);
+                        u[n] = 0;
+                        c->a = authcontext_new_apop(u, NULL, NULL, c->timestamp, digest, inet_ntoa(c->sin.sin_addr));
+                        xfree(u);
+                    }
                 }
 
                 if (c->a) {
@@ -153,7 +162,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                         connection_sendresponse(c, 0, _("Too many authentication attempts."));
 #endif
 
-                        print_log(LOG_ERR, _("connection_do: client `%s' failed to log in after %d attempts"), c->idstr, MAX_AUTH_TRIES);
+                        log_print(LOG_ERR, _("connection_do: client `%s': username `%s': failed to log in after %d attempts"), c->idstr, name, MAX_AUTH_TRIES);
                         return close_connection;
                     } else {
 #ifndef NO_SNIDE_COMMENTS
@@ -161,7 +170,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
 #else
                         connection_sendresponse(c, 0, _("Authentication failed."));
 #endif
-                        print_log(LOG_ERR, _("connection_do: client `%s': %d authentication failures"), c->idstr, c->n_auth_tries);
+                        log_print(LOG_ERR, _("connection_do: client `%s': username `%s': %d authentication failures"), c->idstr, name, c->n_auth_tries);
                         return do_nothing;
                     }
                 }
@@ -191,15 +200,24 @@ enum connection_action connection_do(connection c, const pop3command p) {
 
         /* Do we now have enough information to authenticate using USER/PASS? */
         if (!c->a && c->user && c->pass) {
-            c->a = authcontext_new_user_pass(c->user, c->pass, c->domain, inet_ntoa(c->sin.sin_addr));
-            if (!c->a && append_domain && c->domain && strcspn(c->user, "@%!") == strlen(c->user)) {
-                /* OK, if we have a domain name, try appending that. */
-                char *nn = xmalloc(strlen(c->user) + strlen(c->domain) + 2);
-                strcpy(nn, c->user);
-                strcat(nn, "@");
-                strcat(nn, c->domain);
-                c->a = authcontext_new_user_pass(nn, c->pass, c->domain, inet_ntoa(c->sin.sin_addr));
-                xfree(nn);
+            c->a = authcontext_new_user_pass(c->user, NULL, c->domain, c->pass, inet_ntoa(c->sin.sin_addr));
+            
+            /* Maybe retry authentication with an added or removed domain name. */
+            if (!c->a && (append_domain || strip_domain)) {
+                int n, len;
+                len = strlen(c->user);
+                n = strcspn(c->user, DOMAIN_SEPARATORS);
+                if (append_domain && c->domain && n == len)
+                    /* OK, if we have a domain name, try appending that. */
+                    c->a = authcontext_new_user_pass(c->user, c->user, c->domain, c->pass, inet_ntoa(c->sin.sin_addr));
+                else if (strip_domain && n != len) {
+                    /* Try stripping off the supplied domain name. */
+                    char *u;
+                    u = xstrdup(c->user);
+                    u[n] = 0;
+                    c->a = authcontext_new_user_pass(u, NULL, NULL, c->pass, inet_ntoa(c->sin.sin_addr));
+                    xfree(u);
+                }
             }
 
             if (c->a) {
@@ -212,11 +230,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 c->state = transaction;
                 return fork_and_setuid; /* Code in main.c sends response in case of error. */
             } else {
-                xfree(c->user);
-                c->user = NULL;
-                memset(c->pass, 0, strlen(c->pass));
-                xfree(c->pass);
-                c->pass = NULL;
+                enum connection_action act;
 
                 ++c->n_auth_tries;
                 if (c->n_auth_tries == MAX_AUTH_TRIES) {
@@ -225,17 +239,26 @@ enum connection_action connection_do(connection c, const pop3command p) {
 #else
                     connection_sendresponse(c, 0, _("Too many authentication attempts."));
 #endif
-                    print_log(LOG_ERR, _("connection_do: client `%s' failed to log in after %d attempts"), c->idstr, MAX_AUTH_TRIES);
-                    return close_connection;
+                    log_print(LOG_ERR, _("connection_do: client `%s': username `%s': failed to log in after %d attempts"), c->idstr, c->user, MAX_AUTH_TRIES);
+                    act = close_connection;
                 } else {
 #ifndef NO_SNIDE_COMMENTS
                     connection_sendresponse(c, 0, _("Lies! Try again!"));
 #else
                     connection_sendresponse(c, 0, _("Authentication failed."));
 #endif
-                    print_log(LOG_ERR, _("connection_do: client `%s': %d authentication failures"), c->idstr, c->n_auth_tries);
-                    return do_nothing;
+                    log_print(LOG_ERR, _("connection_do: client `%s': username `%s': %d authentication failures"), c->idstr, c->user, c->n_auth_tries);
+                    act = do_nothing;
                 }
+
+                memset(c->pass, 0, strlen(c->pass));
+                xfree(c->pass);
+                c->pass = NULL;
+
+                xfree(c->user);
+                c->user = NULL;
+                
+                return act;
             }
         } else {
             connection_sendresponse(c, 1, c->pass ? _("What's your name?") : _("Tell me your password."));
@@ -334,7 +357,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 /* That might have taken a long time. */
                 c->idlesince = time(NULL);
                 if (verbose)
-                    print_log(LOG_DEBUG, _("connection_do: client %s: sent %d-line scan list"), c->idstr, nn + 1);
+                    log_print(LOG_DEBUG, _("connection_do: client %s: sent %d-line scan list"), c->idstr, nn + 1);
             }
             break;
 
@@ -347,14 +370,14 @@ enum connection_action connection_do(connection c, const pop3command p) {
                     connection_sendresponse(c, 0, _("That message is no more."));
                 else {
                     char response[64] = {0};
-                    print_log(LOG_INFO, "UIDL %d (hash = %s)", 1 + msg_num, hex_digest(curmsg->hash));
+                    log_print(LOG_INFO, "UIDL %d (hash = %s)", 1 + msg_num, hex_digest(curmsg->hash));
                     snprintf(response, 63, "%d %s", 1 + msg_num, hex_digest(curmsg->hash));
                     connection_sendresponse(c, 1, response);
                 }
             } else {
                 struct indexpoint *J;
                 int nn = 0;
-                print_log(LOG_INFO, "UIDL");
+                log_print(LOG_INFO, "UIDL");
                 connection_sendresponse(c, 1, _("ID list follows:"));
                 for (J = curmbox->index; J < curmbox->index + curmbox->num; ++J) {
                     if (!J->deleted) {
@@ -368,7 +391,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 /* That might have taken a long time. */
                 c->idlesince = time(NULL);
                 if (verbose)
-                    print_log(LOG_DEBUG, _("connection_do: client %s: sent %d-line unique ID list"), c->idstr, nn + 1);
+                    log_print(LOG_DEBUG, _("connection_do: client %s: sent %d-line unique ID list"), c->idstr, nn + 1);
             }
             break;
 
@@ -377,6 +400,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 curmsg->deleted = 1;
                 connection_sendresponse(c, 1, _("Done."));
                 ++curmbox->numdeleted;
+                curmbox->sizedeleted += curmsg->msglength;
             } else
                 connection_sendresponse(c, 0, _("Which message do you want to delete?"));
             break;
@@ -387,7 +411,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                     connection_sendresponse(c, 0, _("That message is no more."));
                 else {
                     if (verbose)
-                        print_log(LOG_DEBUG, _("connection_do: client %s: sending message %d (%d bytes)"),
+                        log_print(LOG_DEBUG, _("connection_do: client %s: sending message %d (%d bytes)"),
                                     c->idstr, msg_num + 1, (int)curmsg->msglength);
                     connection_sendresponse(c, 1, _("Message follows:"));
                     if (!(curmbox)->send_message(curmbox, c->s, msg_num, -1)) {
@@ -397,7 +421,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                     /* That might have taken a long time. */
                     c->idlesince = time(NULL);
                     if (verbose)
-                        print_log(LOG_DEBUG, _("connection_do: client %s: sent message %d"), c->idstr, msg_num + 1);
+                        log_print(LOG_DEBUG, _("connection_do: client %s: sent message %d"), c->idstr, msg_num + 1);
                 }
                 break;
             } else {
@@ -418,7 +442,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 }
                 
                 if (verbose)
-                    print_log(LOG_DEBUG, _("connection_do: client %s: sending headers and up to %d lines of message %d (< %d bytes)"),
+                    log_print(LOG_DEBUG, _("connection_do: client %s: sending headers and up to %d lines of message %d (< %d bytes)"),
                                 c->idstr, arg2, msg_num + 1, (int)curmsg->msglength);
                 connection_sendresponse(c, 1, _("Message follows:"));
 
@@ -429,17 +453,14 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 /* That might have taken a long time. */
                 c->idlesince = time(NULL);
                 if (verbose)
-                    print_log(LOG_DEBUG, _("connection_do: client %s: sent headers and up to %d lines of message %d"), c->idstr, arg2, msg_num + 1);
+                    log_print(LOG_DEBUG, _("connection_do: client %s: sent headers and up to %d lines of message %d"), c->idstr, arg2, msg_num + 1);
                 break;
             }
                 
 
         case STAT: {
                 char response[32] = {0};
-                /* Size here is approximate as we don't strip off the "From "
-                 * headers.
-                 */
-                snprintf(response, 31, "%d %d", curmbox->num, (int)curmbox->st.st_size);
+                snprintf(response, 31, "%d %d", curmbox->num - curmbox->numdeleted, curmbox->totalsize - curmbox->sizedeleted);
                 connection_sendresponse(c, 1, response);
                 break;
             }
@@ -448,6 +469,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 struct indexpoint *i;
                 for (i = curmbox->index; i < curmbox->index + curmbox->num; ++i) i->deleted = 0;
                 curmbox->numdeleted = 0;
+                curmbox->sizedeleted = 0;
                 connection_sendresponse(c, 1, _("Done."));
                 break;
             }
@@ -491,11 +513,11 @@ int connection_start_transaction(connection c) {
     if (!c) return 0;
     
     if (c->a->gid != getgid()) {
-        print_log(LOG_ERR, _("connection_start_transaction: wrong gid"));
+        log_print(LOG_ERR, _("connection_start_transaction: wrong gid"));
         return 0;
     }
     if (c->a->uid != getuid()) {
-        print_log(LOG_ERR, _("connection_start_transaction: wrong uid"));
+        log_print(LOG_ERR, _("connection_start_transaction: wrong uid"));
         return 0;
     }
     
