@@ -8,7 +8,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: stringmap.c,v 1.9 2001/04/21 03:49:24 chris Exp $";
+static const char rcsid[] = "$Id: stringmap.c,v 1.13 2002/03/18 23:47:34 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -22,51 +22,46 @@ static const char rcsid[] = "$Id: stringmap.c,v 1.9 2001/04/21 03:49:24 chris Ex
 #include "util.h"
 
 /* stringmap_new:
- * Allocate memory for a new stringmap.
- */
+ * Allocate memory for a new stringmap. */
 stringmap stringmap_new() {
-    stringmap S = (stringmap)malloc(sizeof(struct _stringmap));
+    stringmap S;
+    
+    S = xcalloc(sizeof *S, 1);
 
-    if (!S) return NULL;
-
-    memset(S, 0, sizeof(struct _stringmap));
     return S;
 }
 
 /* stringmap_delete:
- * Free memory for a stringmap.
- */
+ * Free memory for a stringmap. */
 void stringmap_delete(stringmap S) {
     if (!S) return;
     if (S->l) stringmap_delete(S->l);
     if (S->g) stringmap_delete(S->g);
 
-    free(S->key);
-    free(S);
+    xfree(S->key);
+    xfree(S);
 }
 
 /* stringmap_delete_free:
  * Free memory for a stringmap, and the objects contained in it, assuming that
- * they are pointers to memory allocated by malloc(3).
- */
+ * they are pointers to memory allocated by xmalloc(3). */
 void stringmap_delete_free(stringmap S) {
     if (!S) return;
-    if (S->l) stringmap_delete(S->l);
-    if (S->g) stringmap_delete(S->g);
+    if (S->l) stringmap_delete_free(S->l);
+    if (S->g) stringmap_delete_free(S->g);
 
-    free(S->key);
-    free(S->d.v);
-    free(S);
+    xfree(S->key);
+    xfree(S->d.v);
+    xfree(S);
 }
 
 /* stringmap_insert:
  * Insert into S an item having key k and value d. Returns an existing key
- * or NULL if it was inserted.
- */
+ * or NULL if it was inserted. */
 item *stringmap_insert(stringmap S, const char *k, const item d) {
     if (!S) return 0;
     if (S->key == NULL) {
-        if (!(S->key = strdup(k))) return NULL;
+        S->key = xstrdup(k);
         S->d   = d;
         return NULL;
     } else {
@@ -78,7 +73,7 @@ item *stringmap_insert(stringmap S, const char *k, const item d) {
                 if (S2->l) S2 = S2->l;
                 else {
                     if (!(S2->l = stringmap_new())) return NULL;
-                    if (!(S2->l->key = strdup(k))) return NULL;
+                    S2->l->key = xstrdup(k);
                     S2->l->d   = d;
                     return NULL;
                 }
@@ -86,7 +81,7 @@ item *stringmap_insert(stringmap S, const char *k, const item d) {
                 if (S2->g) S2 = S2->g;
                 else {
                     if (!(S2->g = stringmap_new())) return NULL;
-                    if (!(S2->g->key = strdup(k))) return NULL;
+                    S2->g->key = xstrdup(k);
                     S2->g->d   = d;
                     return NULL;
                 }
@@ -97,8 +92,7 @@ item *stringmap_insert(stringmap S, const char *k, const item d) {
 
 /* stringmap_find:
  * Find in d an item having key k in the stringmap S, returning the item found
- * on success NULL if no key was found.
- */
+ * on success NULL if no key was found. */
 item *stringmap_find(const stringmap S, const char *k) {
     stringmap S2;
     int i;
