@@ -6,7 +6,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: util.c,v 1.10 2001/10/14 01:13:38 chris Exp $";
+static const char rcsid[] = "$Id: util.c,v 1.14 2002/03/22 19:37:56 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -24,29 +24,11 @@ static const char rcsid[] = "$Id: util.c,v 1.10 2001/10/14 01:13:38 chris Exp $"
 #include <unistd.h>
 #include <sys/mman.h>
 
-#include "errprintf.h"
+#include "md5.h"
 #include "util.h"
 
-/* print_log:
- * Because many systems do not have LOG_PERROR, we use a custom function to
- * write an error to the system log, and optionally to standard error as
- * well.
- */
-extern int log_stderr;      /* in main.c */
-
-void print_log(int priority, const char *fmt, ...) {
-    char *s;
-    va_list ap;
-    va_start(ap, fmt);
-    s = verrprintf(fmt, ap);
-    va_end(ap);
-    syslog(priority, "%s", s);
-    if (log_stderr) fprintf(stderr, "%s\n", s);
-}
-
 /* xwrite:
- * Write some data, taking account of short writes and signals.
- */
+ * Write some data, taking account of short writes and signals. */
 ssize_t xwrite(int fd, const void *buf, size_t count) {
     size_t c = count;
     const char *b = (const char*)buf;
@@ -61,8 +43,7 @@ ssize_t xwrite(int fd, const void *buf, size_t count) {
 }
 
 /* daemon:
- * Become a daemon. From `The Unix Programming FAQ', Andrew Gierth et al.
- */
+ * Become a daemon. From `The Unix Programming FAQ', Andrew Gierth et al. */
 int daemon(int nochdir, int noclose) {
     switch (fork()) {
         case 0:  break;
@@ -124,7 +105,7 @@ int write_file(int fd, int sck, size_t msgoffset, size_t skip, size_t msglength,
 
     filemem = mmap(0, length, PROT_READ, MAP_PRIVATE, fd, offset);
     if (filemem == MAP_FAILED) {
-        print_log(LOG_ERR, "write_file: mmap: %m");
+        log_print(LOG_ERR, "write_file: mmap: %m");
         return 0;
     }
 
@@ -150,7 +131,7 @@ int write_file(int fd, int sck, size_t msgoffset, size_t skip, size_t msglength,
 
     errno = 0;
     if (!try_write(sck, "\r\n", 2)) {
-        print_log(LOG_ERR, "write_file: write: %m");
+        log_print(LOG_ERR, "write_file: write: %m");
         munmap(filemem, length);
         return 0;
     }
@@ -172,24 +153,23 @@ int write_file(int fd, int sck, size_t msgoffset, size_t skip, size_t msglength,
         p = q + 1;
     }
     if (munmap(filemem, length) == -1)
-        print_log(LOG_ERR, "write_file: munmap: %m");
+        log_print(LOG_ERR, "write_file: munmap: %m");
     
     errno = 0;
     if (!try_write(sck, ".\r\n", 3)) {
-        print_log(LOG_ERR, "write_file: write: %m");
+        log_print(LOG_ERR, "write_file: write: %m");
         return 0;
     } else return 1;
 
 write_failure:
-    print_log(LOG_ERR, "write_file: write: %m");
+    log_print(LOG_ERR, "write_file: write: %m");
     munmap(filemem, length);
     return 0;
 }
 
 /* parse_uid:
  * Get a user id from a user name or number. Sets u and returns 1 on success,
- * or returns 0 on failure.
- */
+ * or returns 0 on failure. */
 int parse_uid(const char *user, uid_t *u) {
     char *v;
     long l;
@@ -212,8 +192,7 @@ int parse_uid(const char *user, uid_t *u) {
 
 /* parse_gid:
  * Get a group id from a group name or number. Sets g and returns 1 on
- * success, or returns 0 on failure.
- */
+ * success, or returns 0 on failure. */
 gid_t parse_gid(const char *group, gid_t *g) {
     char *v;
     long l;
@@ -234,8 +213,7 @@ gid_t parse_gid(const char *group, gid_t *g) {
 }
 
 /* hex_digest:
- * Make a hex version of a digest.
- */
+ * Make a hex version of a digest. */
 char *hex_digest(const unsigned char *u) {
     static char hex[33] = {0};
     const unsigned char *p;
@@ -248,8 +226,7 @@ char *hex_digest(const unsigned char *u) {
 
 /* unhex_digest:
  * Turn a hex representation of a digest into binary data. Returns 1 on
- * success or 0 on failure.
- */
+ * success or 0 on failure. */
 int unhex_digest(const char *from, unsigned char *to) {
     const char *p;
     unsigned char *q;
@@ -270,6 +247,7 @@ int unhex_digest(const char *from, unsigned char *to) {
     return 1;
 }
 
+#ifndef MTRACE_DEBUGGING
 /* xmalloc:
  * Malloc, and abort if malloc fails. */
 void *xmalloc(size_t n) {
@@ -302,3 +280,55 @@ void *xrealloc(void *w, size_t n) {
 void xfree(void *v) {
     if (v) free(v);
 }
+
+/* xstrdup:
+ * Strdup, aborting on failure. */
+char *xstrdup(const char *s) {
+    char *t;
+    t = xmalloc(strlen(s) + 1);
+    strcpy(t, s);
+    return t;
+}
+#endif  /* !MTRACE_DEBUGGING */
+
+/* md5_digest:
+ * Make an MD5 digest of some data. */
+void md5_digest(const void *v, const size_t n, unsigned char *md5) {
+    MD5_CTX ctx;
+    MD5Init(&ctx);
+    MD5Update(&ctx, (unsigned char*)v, n);
+    MD5Final(md5, &ctx);
+}
+
+/* md5_digest_str:
+ * Make an MD5 digest in a representable format, by default hex, otherwise in
+ * base64. */
+char *md5_digest_str(const void *v, const size_t n, const int base64) {
+    unsigned char md5[16], *p;
+    static char res[33] = {0};
+    char *q;
+    md5_digest(v, n, md5);
+    if (base64) {
+        /* Base 64 encoding per RFC2045 as from LDAP. */
+        const char b64[] = 
+            "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+        for (p = md5, q = res; p < md5 + 16; p += 3, q += 4) {
+            char s[5] = "====";
+#define P(i)    (p + i > md5 + 16 ? *(p + i) : 0)
+            s[0] = b64[P(0) >> 2];
+            s[1] = b64[(P(0) & 0x3 << 4) | (P(1) & 0xf0 >> 4)];
+            if (p + 1 < md5 + 16)
+                s[2] = b64[(P(1) & 0xf << 2) | (P(2) & 0xc0 >> 4)];
+            if (p + 2 < md5 + 16)
+                s[3] = b64[P(2) & 0x3f];
+            strcat(res, s);
+#undef P
+        }
+    } else {
+        /* Conventional hex encoding. */
+        for (p = md5, q = res; p < md5 + 16; ++p, q += 2)
+            sprintf(q, "%02x", (unsigned int)*p);
+    }
+
+    return res;
+}
